# ----------------------------- Load Data ----------------------------------- 
# order the variables as specficied above, otherwise readWorkbook orders index in increasing manner 
order_dec <- order(c(data_sel))
# set length of observations fpr different proxies
sample_data <-  start_rows(spec, instr_sel[2], fake_instr_period = FALSE,by_date=list(start ="1980-01", end="2007-06"))[[1]] 
sample_proxy <-  start_rows(spec, instr_sel[2], fake_instr_period = FALSE,by_date=list(start="1980-01", end="2007-06"))[[2]]
# Load data
series_sheet1 <- as.matrix(openxlsx::readWorkbook("../../data/VAR_data.xlsx",sheet="VAR_data", colNames = FALSE, rows=sample_data, cols=data_sel))[,order_dec]
# Load instrument
instrument_data <-   as.matrix(openxlsx::readWorkbook("../../data/factor_data.xlsx",sheet="factor_data", colNames = FALSE,  rows=sample_proxy,cols=instr_sel))
# add instrument name to data names
names <- c(data_names(data_sel, 1), "Instrument")
# create date series
date_var <- substr(as.Date(as.yearmon(as.Date(as.numeric(series_sheet1[,1]),origin="1899-12-30"))),1,7)
date_ins <- substr(as.Date(as.yearmon(as.Date(as.numeric(instrument_data[,1]),origin="1899-12-30"))),1,7)
date_list <- list(date_var=date_var, date_ins=date_ins)
# cut date column
y <- as.matrix(cbind(series_sheet1[,-1]))
spec$instrument <- as.matrix(instrument_data[,-1])

# transform data
y <- as.matrix(transform_data(y, code))
y[,which(code==3)] <- y[,which(code==3)]*100

# also crop instrument, if transformation kills one observation
if (any(code %in% c(2,3))){
  if (is.matrix(spec$instrument)){
    spec$instrument<- spec$instrument[-1,]
  } else{
    spec$instrument<- spec$instrument[-1]
    
  }
  date_ins <- date_ins[-1]
  date_var <- date_var[-1]
  date_list <- list(date_var=date_var, date_ins=date_ins)
}

# standardize data
if (stand == TRUE){
  stand_fac <- apply(y, 2, function(x) std(x))
  y <- apply(y, 2, function(x) (x - mean(x))/std(x))
  if (is.matrix(spec$instrument)){
    spec$instrument <- apply(spec$instrument, 2, function(x) (x - mean(x, na.rm=TRUE))/sd(x, na.rm=TRUE))    
  } else {
    spec$instrument <- (spec$instrument - mean(spec$instrument))/sd(spec$instrument) 
  }
}

# augment VAR model with instrument (PH-VAR)
if (aug_var==TRUE){
  y <- cbind(y, spec$instrument)
}
y_full <- y

# Get Sample Size and number of variables
Samp <-  dim(y)[1] # Sample Size
K <-  dim(y)[2] # number of variables in system
# Lag data and merge to Z-matrix
y_lag <- y_lag_cr(y, lag_length)$lags
y_start <- (y[(1:lag_length),])
y <- as.matrix(y[-(1:lag_length),])
#generate Z matrix with constant
con <- matrix(1,Samp-lag_length,1)
if (spec$trend==TRUE){
  trend <- seq(1,Samp-lag_length)
  Z <- cbind(con, trend,y_lag)
  rm(trend)
} else {
  Z <- cbind(con, y_lag)
  
}
rm(con)

#  Generate the recession and economic events data
recessions.trim <- events(date_var, recession = TRUE) 
uncert.trim <- events(date_var, recession = FALSE)

#  logging
log_path <- "log_ms.txt" # where to log
file.create(log_path)