rm(list = ls()) # To clear all
# set path
setwd("D://het_instrument/Code/estimation/main")

# ------------------ Load packages and source functions ------------------
# load packages and source functions
source("./general/organization_code/load_packages.R")
source("./general/source.R")
sourceFolder("./general", FALSE, FALSE)
sourceFolder("../ms_switch/", FALSE, FALSE)
set.seed(4673590)
# set up environments to source to
# set up parallel
source("./general/organization_code/setparallel.R")
# ----------------------------- Specification of Model -----------------------------------
# Load the (economic) data (not the instrument yet!) from file "./data/VAR_data.xlsx" by selecting various column indices below.
data_sel <-  c(1, 3, 4, 32, 18) # refers to columns in excel sheet: Fed Funds Rate, Industrial Production (in logs), Price index of PCE (in logs), Raw Materials Price index (no transformation)
aug_var <- TRUE # augment VAR model with instrument?
# Load the instrument data from file "./data/factor_data.xlsx" by selecting the number of a column index below.
# index 10 is RR (long) instrument, which is the baseline of the paper.
# index 3 is GK instrument, index 11 is Miranda-Agrippino and Rossi, index 12 is Benanke and Mihov, index 13 is Barachian and Crowe, 14 is factor of all instruments
instr_sel <- c(1,10) # Romer and Romer
# Transformation
code <- c(0,0,0,1,0) # 0 = No transformation, 1 = Logs, 2 = Log Diff, 3 = Differences 
lag_length <- 6 # number of lags

# attach names (data, shock, instrument)
names <- c(data_names(data_sel, 1))
shock_names <-  paste0(rep("Shock ", length(names)), c(1:length(names)) )
instr_names <- paste0("Instrument: ", data_names(instr_sel, 2))
names
instr_names
states <- 2 # States of Markov Chain
horizon_imp <- 60 # Horizon to be used in impulse responses
boot_rep <- 1000 # Bootstrap repetitions (1000 used in paper, causes pretty long run-time later)  
LM <- rep(seq(1,10,1),100) # starting value for Lambda, see following comment.
############################################## Note on starting values ##################################################################
# In the paper we estimate MS-models with 1000 different starting values (for Lambda and D --> see the appendix of the paper for the exact procedure) to assure, that the maximum of the likelihood is found. 
# This procedure consumes enormous computational capacities (we ran the estimation on a server with High-Performance Computing (HPC) capacities). For the sake of being able to replicate or results, in what follows below,
# for the respective MS-models we have implemented the starting values for Lambda resulting from our large-scale calculations (for the Romer-Romer baseline case!). As the starting values for the D-matrix are stochastic, the test results of our LR-testing 
# procedure, for example, vary slightly from the results in the paper. For exact replication of the results, the models must be run with all 1000 starting values.
#########################################################################################################################################
# Put all information in spec-list element which is passed to the algorithm below
# spec$unres refers to an unidentified model, i.e. the restriction B*Lambda*B does not hold. One simply estimates the covariances Sigma1, Sigma2..., etc.
spec <- list(ins_dim=1, trend = FALSE, states = states, const = matrix(1,states,1), iterations = length(LM), lag = lag_length, horizon = horizon_imp, rep = boot_rep, boot = "yes", instrument = TRUE, unres = FALSE, sim = FALSE)
stand <-  TRUE # Standardize data?
# This part of code loads data
source("./general/organization_code/data_prep.R")
# specification of instrument in augmented model
spec$clean <- FALSE # cleaned instrument? FALSE means zeros are implemented (restricted estimation --> no "cleansing", see res_type option), TRUE means estimation of parameters
spec$res_type <- "row" # how to clean the instrument? Options are "row" to restrict only influence of other variables,  "column" to restrict only autoregressive influence, "all" restriction on other variables and autoregressive 
spec$ins_drive <-  TRUE # does instrument contribute to states?
################### SPECIFICATION OF DIFFERENT MODELS
# spec$restric is relevant container that decides on spefication of MS-Model
# the options are
# 1) h-var-res = heteroskedastic VAR (with zeros in rows and cols of D-matrix for instrument) 
# 2) ph-var = het. proxy var (use spec$lenb to specify number elements of beta vector)
# 3) invarB-res = zeros on last column of D
# 4) invarB = no restrictions on D-matrix 

############################################ H-VAR ##############################################################
spec$restric <- "h-var-res" 
spec$boot <- "yes"  
LM <- rep(seq(1,10,1),100)[49] # starting values for Lambda were determined running 1000 different tries as described in Appendix "Notes on computations" to this paper, therefore results here may differ slightly from the results in the paper.
spec$iterations <- length(LM)
h_var <- ms_var(spec, y, Z, LM, Samp, K, y_start, log_path, code, verbose=FALSE, start_val=c(),aug_var=TRUE, b_method="rnd")
h_var$ll_ms

############################################ PH-VAR ############################################################
spec$lenb <- 1 # number of elements to estimate in last row of B (the beta vector!)
spec$restric <- "ph-var"
LM <- rep(seq(1,10,1),100)[89] # starting values for Lambda were determined running 1000 different tries as described in Appendix "Notes on computations" to this paper, therefore results here may differ slightly from the results in the paper.
ph_var <-  ms_var(spec, y=y, Z=Z, LM, Samp, K=K, y_start = y_start, log_path, code=code, date_list, verbose=FALSE, start_val=c(), aug_var, b_method="rnd")
ph_var$ll_ms
# LR-Test for strength of instrument
test_strength <- lr_test(ph_var$ll_ms, h_var$ll_ms, df=1)
test_strength

############################################# INVAR-B-RESTRICTED ################################################
spec$restric <- "invarB-res"
spec$boot <- "no"
LM <- rep(seq(1,10,1),100)[20] # starting values for Lambda were determined running 1000 different tries as described in Appendix "Notes on computations" to this paper, therefore results here may differ slightly from the results in the paper.
invarB_res <- ms_var(spec, y=y, Z=Z, LM, Samp, K=K, y_start = y_start, log_path, code=code, date_list, verbose=FALSE, start_val=c(),aug_var, b_method="rnd")
invarB_res$ll_ms
# LR-Test for endogeneity of instrument
test_endo <- lr_test(invarB_res$ll_ms, ph_var$ll_ms, df=K-2)
test_endo

############################################ STATE-INVARIANT B ##################################################
spec$restric <- "invarB"
LM <- rep(seq(1,10,1),100)[35] # starting values for Lambda were determined running 1000 different tries as described in Appendix "Notes on computations" to this paper, therefore results here may differ slightly from the results in the paper.
spec$boot <- "no"
invarB <- ms_var(spec, y=y, Z=Z, LM, Samp, K=K, y_start = y_start, log_path, code=code, date_list, verbose=FALSE, start_val=c(),aug_var, b_method="rnd")

save.image("results.RData") # save results

############################################ Reproduce empirical baseline results ##################################################
# Order of results as they appear in paper.

# Table 8, line 3:
invarB$ll_ms # Likelihood of model
# Table 9:
invarB$Sigma # Reduced form covariance matrix
# Table 10:
diag(diag(invarB$Lambda[6:10,])[c(5,3,2,1,4)]) # Estimates of Lambda
invarB$se_struc[26:30][c(5,3,2,1,4)] # Standard errors

# Figure 1:
plot_state_prob(state_prob = invarB$KsiT, recessions.trim, uncert.trim = c(), date_var, path="")
# Table 11:

table11_out <- matrix(c(test_endo$LR_stat, 3, test_endo$p_LR, test_strength$LR_stat, 1, test_strength$p_LR), 3,2)
colnames(table11_out) <- c("Exogeneity", "Relevance")
rownames(table11_out) <- c("LR stat", "Degrees of fr.","p-value")
table11_out

# Figure 2: (different ordering of variables and columns and signs potentially)
source("./general/organization_code/fig_2.R")

############################################ Proxy VAR ##########################################################
# Run (at best in separate R-Session) the file "main_empirical_proxy.R" to estimate and save results of a homesekdastic Proxy VAR
# Load results of this estimation
pr_var <- readRDS(file="pr_var.Rds")
#################################################################################################################

# Figure 3: (different ordering of variables and columns and signs potentially)
source("./general/organization_code/fig_3.R")
