bootstrap_ms_block <- function(est_ms, lag_length, spec, Samp,K, y, y_start, LM, log_path, verbose){
  y_true <-  y
  horizon <- spec$horizon
  rep <-  spec$rep
  B <- est_ms$B
  Theta <- est_ms$Theta
  u_est <- est_ms$u_est
  
  if (Samp <= 250){
    l <- 20
  } else if (Samp > 250){
    l <- 40
  }
  
  s <-  (Samp-lag_length)/l   
  y_comp <- as.matrix(rbind(y_start, y))
  # create empty arrays to hold the data
  if (spec$trend == FALSE){
    Z_star_array <- array(data=NA, dim=c( (Samp-lag_length),(K*lag_length+1),rep))
  } else {
    Z_star_array <- array(data=NA, dim=c( (Samp-lag_length),(K*lag_length+2),rep))
  }
  y_star_array <- array(data=NA, dim=c((Samp-lag_length),K,rep))
  u_star_array <- array(data=NA, dim=c((Samp-lag_length),K,rep))
  
  # array if instrument approach is supposed to be used 
  boot_data <- block_boot_data(s,l,rep, K, Samp, lag_length, u_est, y_start, y_comp, Theta, u_star_array,Z_star_array,y_star_array)
  #re-run the models, with estimated parameters (no grid search)
  results_boot <- vector(mode = "list", length = rep)
  # re-run the model 
  # ------------------------------------------------------------------------------------
  boot_imp_complete <- array(data=NA, dim=c(rep,horizon+1 ,K^2))
  boot_imp_cum_complete <- array(data=NA, dim=c(rep,horizon+1 ,K^2))
  results_boot <- foreach(n=1:rep,.verbose =FALSE, .export=ls(ms_envir), .packages=c("nloptr","matrixcalc", "pracma","miscTools", "MASS", "Rsolnp")) %dopar% { # loop through the grid
    #-------------------------------- data ------------------------------------------------
    y  <- as.matrix(boot_data$y_star_array[,,n])
    Z  <- as.matrix(boot_data$Z_star_array[,,n])
    #-----------------------------ML-estimation----------------------------------------------
    param_condml <-  list(B= est_ms$B, Lambda=est_ms$Lambda, Ksi=est_ms$Ksi, P=est_ms$P, KsiT=est_ms$KsiT)
    cat(paste("Bootstrap iteration:", n ,"\r\n"),  file=log_path, append=TRUE)
    if (spec$clean == FALSE){ # if there is an instrument in the model, automatically restrict its impact by Restricted OLS
      results_boot[[n]] <- estimate_ms_condml_res(Samp,K,y,Z,spec,LM,param=param_condml)
    } else if (spec$clean == TRUE){ # this is the regular Expectation-Maximization algorithm, if no instrument is applied
      results_boot[[n]] <- estimate_ms_condml(Samp,K,y,Z,spec,LM,param=param_condml)
    }
    while (is.null(results_boot[[n]])){ # in case of singularities draw new data and repeat bootstrap replication
      boot_data <- create_boot_data(y_true, y_start, lag_length, spec, Samp, Theta, K, u_est)  
      y_star_array[,,n] <- boot_data[[1]]
      Z_star_array[,,n] <- boot_data[[2]]
      #   #-------------------------------- data ------------------------------------------------
      y  <- as.matrix(y_star_array[,,n])
      Z  <- as.matrix(Z_star_array[,,n])
      #-----------------------------ML-estimation----------------------------------------------  
      param_condml <-  list(B= est_ms$B, Lambda=est_ms$Lambda, Ksi=est_ms$Ksi, P=est_ms$P, KsiT=est_ms$KsiT)
      cat(paste("Bootstrap iteration:", n ,"\r\n"),  file=log_path, append=TRUE)
      if (spec$clean == FALSE){ # if there is an instrument in the model, automatically restrict its impact by Restricted OLS
        results_boot[[n]] <-  tryCatch({estimate_ms_condml_res(Samp,K,y,Z,spec,LM,param=param_condml)}, error = function(e) NULL)
      } else if (spec$clean == TRUE){ # this is the regular Expectation-Maximization algorithm, if no instrument is applied
        results_boot[[n]] <-  tryCatch({estimate_ms_condml(Samp,K,y,Z,spec,LM,param=param_condml)}, error = function(e) NULL)
      }
    }
    results_boot[[n]] <- results_boot[[n]] # this must be the last line in a foreach %dopar% operation
  }
  horizon_2 <- dim(boot_imp_complete[,,1])[2]
  no_shocks <- length(boot_imp_complete[1,1,])
  # create empty arrays  
  lower_5 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_5 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  lower_10 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_10 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  
  lower_68 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_68 <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  lower_5_cum <-array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_5_cum <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  lower_10_cum <-array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_10_cum <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  
  lower_68_cum <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  upper_68_cum <- array(NA, dim=c(no_shocks, horizon_2,spec$states))
  interval1 <- 0.025
  interval2 <- 0.05
  interval3 <- 0.16
  
  for (m in 1:spec$states){
    for (n in 1:rep){
      for (l in 1:(K^2)){
        boot_imp_complete[n,,l] <- results_boot[[n]]$imp_rep[l,,m]
        boot_imp_cum_complete[n,,l] <- results_boot[[n]]$imp_rep_cum[l,,m]
      } # end of loop through l (number of shocks)
    } # end of loop through n (rep)
    
    # sort the bootstrapped impulse responses and order them by size
    sorted_impulse <- boot_imp_complete
    sorted_impulse_cum <- boot_imp_cum_complete
    
    # # order by size 
    for (i in 1:no_shocks){
      sorted_impulse[,,i] <- apply(boot_imp_complete[,,i], 2, sort, decreasing=TRUE)
      sorted_impulse_cum[,,i] <- apply(boot_imp_cum_complete[,,i], 2, sort, decreasing=TRUE)
      lower_5[i,,m] <- sorted_impulse[ceiling((1-interval1)*rep),,i]
      upper_5[i,,m] <- sorted_impulse[floor(interval1*rep),,i]
      lower_10[i,,m] <- sorted_impulse[ceiling((1-interval2)*rep),,i]
      upper_10[i,,m] <- sorted_impulse[floor(interval2*rep),,i]
      lower_68[i,,m] <- sorted_impulse[ceiling((1-interval3)*rep),,i]
      upper_68[i,,m] <- sorted_impulse[floor(interval3*rep),,i]
      lower_5_cum[i,,m] <- sorted_impulse_cum[ceiling((1-interval1)*rep),,i]
      upper_5_cum[i,,m] <- sorted_impulse_cum[floor(interval1*rep),,i]
      lower_10_cum[i,,m] <- sorted_impulse_cum[ceiling((1-interval2)*rep),,i]
      upper_10_cum[i,,m] <- sorted_impulse_cum[floor(interval2*rep),,i]
      lower_68_cum[i,,m] <- sorted_impulse_cum[ceiling((1-interval3)*rep),,i]
      upper_68_cum[i,,m] <- sorted_impulse_cum[floor(interval3*rep),,i]
    }
  }
  out <-list(upper_int5 = upper_5, lower_int5 =lower_5,upper_int10 = upper_10, lower_int10 =lower_10,upper_int68 = upper_68, lower_int68 =lower_68, sorted_impulse=sorted_impulse, upper_int5_cum = upper_5_cum, lower_int5_cum =lower_5_cum,upper_int10_cum = upper_10_cum, lower_int10_cum =lower_10_cum,upper_int68_cum = upper_68_cum, lower_int68_cum =lower_68_cum, sorted_impulse_cum=sorted_impulse_cum)
  return(out)
}

