estimate_ms_condml <- function(Samp,K,y,Z,spec,LM,param){
  Tolvalue <-  10^-9
  # See in paper of Herwartz and Lütkepohl (2011) pp 19ff. an explanation of the algorithm.  
  #--------------------------------------------------------------------------------------------
  ################
  # EM Algorithm #      
  ################
  #--------------------------------------------------------------------------------------------
  ##########################
  # Set initial parameters #
  ##########################
  # initial tranistion matrix
  P <-  param[[4]]
  y_reg <- c(t(y)) # transpose and vectorize data 
  # reestimate te slope coefficients
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  # initial coefficient vector
  Theta <-  solve(A1)%*%A2
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, K, K*spec$lag+1)
  u_est <- residuals3(y,Theta_mat,Z)  
  B <-  param[[1]]
  Lambda <- param[[2]]
  # initial KSI 
  Ksi <- param[[3]]
  KsiT <- param[[5]]
  # Calculate the sum of probabilities
  Tm <- t(as.matrix(colSums(KsiT)- KsiT[1,]))
  # Form initial covariance matrices
  Sigma <- matrix(NA,K*spec$states,K)
  for (position in  1:spec$states){
    Sigma[((1+K*(position-1)):(K*position)), ]  <- B%*%Lambda[((1+K*(position-1)):(K*position)), ]%*%t(B)
  }
  # Round is a counter to accumulate number of iterations 
  # Estimate until convergence is achieved
  ##########################
  #   Maximization Step    #
  ##########################
  
  # Estimate the likelihood function and retrieve matrices B and Lambdas, BL stores B first, then it has diagonal entries of lambdas
  val <-  matrix(0, 1,2)
  val[1,1] <- 1
  val[1,2] <- 1
  RND <- 1
  
  # start_time <- proc.time()
  while ((val[RND,2] > 10^-6) & (RND<100) ){
    u_est <- residuals4(y, Theta[,RND],Z, K, spec)
    out_min <- minimize_ms_condml(KsiT,Samp,K,Tm,Lambda,B, spec,u_est, Theta)
    val_help <- abs( (out_min$val - val[RND,1])/val[RND,1] )
    val_help2 <- cbind(out_min$val, val_help)
    val <- rbind(val, val_help2)
    rm(val_help,val_help2)
    # Store B and Lambda
    # turn into matrix
    out_min$B <- matrix(out_min$B,K,K)
    # store B
    B <-  out_min$B[,(1:K)]
    # Calculate the covariance matrices Sigma_m (for all different Lambda_m) from decomposition of B and lambda
    eigen_contr <- c() # if eigenvalues of Sigma smaller than threshold, break the loop and return with no result
    for (position in 1:spec$states){
      Sigma[(1+K*(position-1)):(K*position),] <- B%*%  Lambda[(1+K*(position-1)):(K*position),]%*%t(B)
      eigen_contr <-  c(eigen_contr, eigen(Sigma[(1+K*(position-1)):(K*position),])$values)
    }
    
    # Estimate parameter vector Theta (add up over the different states)
    T2 <- matrix(0, (K*spec$lag+1)*K, (K*spec$lag+1)*K)
    T3 <- matrix(0, (K*spec$lag+1)*K,(Samp-spec$lag)*K)
    for (position in 1:spec$states){
      T2 <- T2 + kronecker( (t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]) %*%Z),solve(get_sigma(Sigma,K,position)))
      T3 <- T3 + kronecker(t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]),solve(get_sigma(Sigma,K,position)))
    }
    Theta_help <- solve(T2)%*%T3%*%c(y_reg)
    
    Theta <- cbind(Theta, Theta_help )
    rm(Theta_help)
    RND <- RND +1
  } # end of second while loop
  ####################################
  if (RND <= 100){
    convergence <- 1
  } else if (RND > 100){
    convergence <- 0
  }
  
  # order the Lambdas by size of the lambda_i,j of the first state
  # order the Lambdas by size of the lambda_i,j of the first state
  order <- order(out_min$B[(K^2+1):( K^2+K)], decreasing = FALSE)
  # store lambda
  for (position in 2:spec$states){
    Lambda[(1+K*(position-1)):(K*position),] <- diag(diag(Lambda[(1+K*(position-1)):(K*position),])[order] ,K,K)
  }
  B <-  B[,order]
  # extract Theta from matrix of Thetas (ROUND*RND = RND_ALL)
  ThetaLast <- matrix(Theta[,RND],K, 1+spec$lag*K)
  # num_param <- dim(out_min$BL)[1]*dim(out_min$BL)[2] + dim(ThetaLast)[1]*dim(ThetaLast)[2]
  IterationLik <- val[RND,1]
  
  # order column of zeros last
  
  if (spec$restric == "h-var-res" ||  spec$restric == "ph-var" ||  spec$restric == "h-var" || spec$restric == "invarB-res"){
    col_last <- which.min(abs(apply(B, 2, function(x) sum(x[1:(K-1)]))))
    cols <- c(1:K)[-col_last ]
    order <- c(cols, col_last)
    B <- B[,order]
    lambda_vec <- c()
    for (position in 2:spec$states){
      Lambda[(1+K*(position-1)):(K*position),] <- diag(diag(Lambda[(1+K*(position-1)):(K*position),])[order] ,K,K)
      lambda_vec <- c(lambda_vec,diag(Lambda[(1+K*(position-1)):(K*position),]))
    }
  }
  
  # Using Likelihood Preserving normalization (Waggoner/Zha (2003)):
  for (i in  1:K){
    if (t(diag(K)[,i])%*%solve(B)%*%B[,i] < 0){
      B[,i] <-  -1*B[,i]
    }
  }
  
  # Impulse responses
  imp_rep <- calc_imp_response(B, ThetaLast, Lambda, spec$horizon, K, spec, spec$lag)
  
  out <- list(Theta =ThetaLast, Sigma = Sigma,KsiT=KsiT, P =P, logL=IterationLik, B=B, Lambda=Lambda, Ksi=Ksi, convergence =convergence, imp_rep =imp_rep$impulse, imp_rep_cum = imp_rep$impulse_cum, Phi_coef = imp_rep$Phi)
  return(out)
  
}