estimate_ms_condml_res <- function(Samp,K,y,Z,spec,LM,param){
  Tolvalue <-  10^-9
  # See in paper of Herwartz and Lütkepohl (2011) pp 19ff. an explanation of the algorithm.  
  #--------------------------------------------------------------------------------------------
  ################
  # EM Algorithm #      
  ################
  #--------------------------------------------------------------------------------------------
  ##########################
  # Set initial parameters #
  ##########################
  # initial tranistion matrix
  P <-  param[[4]]
  y_reg <- c(t(y)) # transpose and vectorize data 
  # reestimate te slope coefficients
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  Theta <-  solve(A1)%*%A2
  #### Restricted Estimator OLS ########
  # restriction are always placed on the last variable (the instrument)
  # vector of positions of restrictions, index value of restricted coefficient in vectorized coefficient matrix
  if (spec$trend ==FALSE){
    if (spec$res_type=="all"){
      pos_res <- c(K,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))) 
                   ,c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-1))))
    } else if (spec$res_type=="row"){
      pos_res <- c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="col"){
      pos_res <- c(K,(1:(K)*K+K),apply(matrix((1:(K)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))))
    } else if (spec$res_type=="diag"){
      pos_res <- c((1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } 
    # difference to pos_res --> to define mtrix R as specified in Lütkepohl book page 194 
    rest <- setdiff(1:(K^2*spec$lag+K),sort(pos_res))
    R <-   matrix(0, K^2*spec$lag+K,K^2*spec$lag+K-length(pos_res))
    for (j in 1:length(rest)){
      R[rest[j],j] <- 1
    }
    r <- rep(0,K*(K*spec$lag+1))
    
  } else {
    if (spec$res_type=="all"){
      pos_res <- c(K,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))) 
                   ,c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-1))))
    } else if (spec$res_type=="row"){
      pos_res <- c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="col"){
      pos_res <- c(K,(1:(K)*K+K),apply(matrix((1:(K)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))))
    } else if (spec$res_type=="diag"){
      pos_res <- c((1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } 
    # difference to pos_res --> to define mtrix R as specified in Lütkepohl book page 194 
    rest <- setdiff(1:(K^2*spec$lag+K),sort(pos_res))
    R <-   matrix(0, K^2*spec$lag+2*K,K^2*spec$lag+2*K-length(pos_res))
    for (j in 1:length(rest)){
      R[rest[j],j] <- 1
    }
    r <- rep(0,K*(K*spec$lag+2))
  }
  
  z <- c(y_reg)- kronecker(Z, diag(K))%*%r
  gamma <- solve(t(R)%*%(kronecker(t(Z)%*%Z, diag(K)))%*%R)%*%t(R)%*%(kronecker(t(Z),diag(K)))%*%z
  Theta <- R%*%gamma
  
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, nrow=K)
  u_est <- residuals3(y,Theta_mat,Z) 
  B <-  param[[1]]
  Lambda <- param[[2]]
  Ksi <- param[[3]]
  KsiT <- param[[5]]
  # Calculate the sum of probabilities
  Tm <- t(as.matrix(colSums(KsiT)- KsiT[1,]))
  # Form initial covariance matrices
  Sigma <- matrix(NA,K*spec$states,K)
  for (position in  1:spec$states){
    Sigma[((1+K*(position-1)):(K*position)), ]  <- B%*%Lambda[((1+K*(position-1)):(K*position)), ]%*%t(B)
  }
  # Round is a counter to accumulate number of iterations 
  # Estimate until convergence is achieved
  ##########################
  #   Maximization Step    #
  ##########################
  
  # Estimate the likelihood function and retrieve matrices B and Lambdas, BL stores B first, then it has diagonal entries of lambdas
  val <-  matrix(0, 1,2)
  val[1,1] <- 1
  val[1,2] <- 1
  RND <- 1
  
  # start_time <- proc.time()
  while ((val[RND,2] > 10^-6) & (RND<100) ){
    u_est <- residuals4(y, Theta[,RND],Z, K, spec)
    out_min <- minimize_ms_condml(KsiT,Samp,K,Tm,Lambda,B, spec,u_est, Theta)
    val_help <- abs( (out_min$val - val[RND,1])/val[RND,1] )
    val_help2 <- cbind(out_min$val, val_help)
    val <- rbind(val, val_help2)
    rm(val_help,val_help2)
    # Store B and Lambda
    # turn into matrix
    out_min$B <- matrix(out_min$B,K,K)
    # store B
    B <-  out_min$B[,(1:K)]
    # Calculate the covariance matrices Sigma_m (for all different Lambda_m) from decomposition of B and lambda
    eigen_contr <- c() # if eigenvalues of Sigma smaller than threshold, break the loop and return with no result
    for (position in 1:spec$states){
      Sigma[(1+K*(position-1)):(K*position),] <- B%*%  Lambda[(1+K*(position-1)):(K*position),]%*%t(B)
      eigen_contr <-  c(eigen_contr, eigen(Sigma[(1+K*(position-1)):(K*position),])$values)
    }
    # Estimate parameter vector Theta (add up over the different states, see Appendix (page 21) Herwartz/Lütkepohl 2011 WP)
    T2 <- matrix(0, dim(R)[2], dim(R)[2])
    T3 <- matrix(0, dim(R)[2],(Samp-spec$lag)*K)
    # Sigma <- Sigma[-c(3,6),]
    # Sigma <- Sigma[,-c(3)]
    
    for (position in 1:spec$states){
      T2 <- T2 + t(R)%*%kronecker( (t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]) %*%Z),solve(get_sigma(Sigma,K,position)))%*%R
      T3 <- T3 + t(R)%*%kronecker(t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]),solve(get_sigma(Sigma,K,position)))
    }
    # Theta <- solve(T2)%*%T3%*%c(y_reg)
    gamma_help <- solve(T2)%*%T3%*%c(z)
    Theta_help <- R%*%gamma_help
    Theta <- cbind(Theta, Theta_help )
    rm(Theta_help)
    RND <- RND +1
  } # end of second while loop
  ####################################
  if (RND <= 100){
    convergence <- 1
  } else if (RND > 100){
    convergence <- 0
  }
  # extract Theta from matrix of Thetas (ROUND*RND = RND_ALL)
  ThetaLast <- matrix(Theta[,RND],K)
  # num_param <- dim(out_min$BL)[1]*dim(out_min$BL)[2] + dim(ThetaLast)[1]*dim(ThetaLast)[2]
  IterationLik <- val[RND,1]
  # Using Likelihood Preserving normalization (Waggoner/Zha (2003)):
  for (i in  1:K){
    if (t(diag(K)[,i])%*%solve(B)%*%B[,i] < 0){
      
      B[,i] <-  -1*B[,i]
    }
  }
  # Impulse responses
  imp_rep <- calc_imp_response(B, ThetaLast, Lambda, spec$horizon, K, spec, spec$lag)
  
  out <- list(Theta =ThetaLast, Sigma = Sigma,KsiT=KsiT, P =P, logL=IterationLik, B=B, Lambda=Lambda, Ksi=Ksi, convergence =convergence, imp_rep =imp_rep$impulse, imp_rep_cum = imp_rep$impulse_cum, Phi_coef = imp_rep$Phi)
  return(out)
}