estimate_ms_condmlblock_res <- function(Samp,K,y,Z,spec,LM,param){
  Tolvalue <-  10^-6
  verbose = FALSE
  # See in paper of Herwartz and Lütkepohl (2011) pp 19ff. an explanation of the algorithm.  
  #--------------------------------------------------------------------------------------------
  ################
  # EM Algorithm #      
  ################
  #--------------------------------------------------------------------------------------------
  ##########################
  # Set initial parameters #
  ##########################
  # initial tranistion matrix
  P <-  param[[4]]
  y_reg <- c(t(y)) # transpose and vectorize data 
  # reestimate te slope coefficients
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  Theta <-  solve(A1)%*%A2
  #### Restricted Estimator OLS ########
  # restriction are always placed on the last variable (the instrument)
  # vector of positions of restrictions, index value of restricted coefficient in vectorized coefficient matrix
  if (spec$trend ==FALSE){
    if (spec$res_type=="all"){
      pos_res <- c(K,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))) 
                   ,c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-1))))
    } else if (spec$res_type=="row"){
      pos_res <- c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="col"){
      pos_res <- c(K,(1:(K)*K+K),apply(matrix((1:(K)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))))
    } else if (spec$res_type=="diag"){
      pos_res <- c((1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="test"){
      pos_res <- c(36,52,68,84,100,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
      
    }
    # difference to pos_res --> to define mtrix R as specified in Lütkepohl book page 194 
    rest <- setdiff(1:(K^2*spec$lag+K),sort(pos_res))
    R <-   matrix(0, K^2*spec$lag+K,K^2*spec$lag+K-length(pos_res))
    for (j in 1:length(rest)){
      R[rest[j],j] <- 1
    }
    r <- rep(0,K*(K*spec$lag+1))
    
  } else {
    if (spec$res_type=="all"){
      pos_res <- c(K,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))) 
                   ,c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-1))))
    } else if (spec$res_type=="row"){
      pos_res <- c(apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="col"){
      pos_res <- c(K,(1:(K)*K+K),apply(matrix((1:(K)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))))
    } else if (spec$res_type=="diag"){
      pos_res <- c((1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
    } else if (spec$res_type=="test"){
      pos_res <- c(36,52,68,84,100,(1:(K-1)*K+K),apply(matrix((1:(K-1)*K+K)), 1, function(x) x +cumsum( rep(K^2,spec$lag-1))), apply(outer(K*K, 1:spec$lag)+1,2, function(x) seq(x,x+K-2)))
      
    }
    
    # difference to pos_res --> to define mtrix R as specified in Lütkepohl book page 194 
    rest <- setdiff(1:(K^2*spec$lag+K),sort(pos_res))
    R <-   matrix(0, K^2*spec$lag+2*K,K^2*spec$lag+2*K-length(pos_res))
    for (j in 1:length(rest)){
      R[rest[j],j] <- 1
    }
    r <- rep(0,K*(K*spec$lag+2))
  }
  
  z <- c(y_reg)- kronecker(Z, diag(K))%*%r
  gamma <- solve(t(R)%*%(kronecker(t(Z)%*%Z, diag(K)))%*%R)%*%t(R)%*%(kronecker(t(Z),diag(K)))%*%z
  Theta <- R%*%gamma
  
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, nrow=K)
  u_est <- residuals3(y,Theta_mat,Z) 
  B <-  param[[1]]
  Lambda <- param[[2]]
  Ksi <- param[[3]]
  KsiT <- param[[5]]
  Eta <- matrix(NA,(Samp-spec$lag),spec$states)
  # Calculate the sum of probabilities
  Tm <- t(as.matrix(colSums(KsiT)- KsiT[1,]))
  # Form initial covariance matrices
  Sigma <- matrix(NA,K*spec$states,K)
  for (position in  1:spec$states){
    Sigma[((1+K*(position-1)):(K*position)), ]  <- B%*%Lambda[((1+K*(position-1)):(K*position)), ]%*%t(B)
  }
  # Round is a counter to accumulate number of iterations 
  # Initialize the likelihood values
  LogValue  <- matrix(NA, 1,3)
  LogValue[1,1] <- 0
  LogValue[1,2] <- 1
  LogValue[1,3] <- 1
  RND_ALL <- 1
  # Estimate until convergence is achieved
  # Counter that accounts for number of iterations (ROUND (first while loop)) and number of iterations within each round (RND (second while loop))
  ROUND <- 1
  
  while((abs(LogValue[ROUND,3]) > Tolvalue)  & (ROUND<=500)){
    
    ##########################
    #     Expectation step   #
    ##########################
    
    # Calculate the conditional distribution function Eta
    # In the first round Eta needs to be created 
    # In the next rounds Eta exists and only needs to be updated.
    # Function get_sigma retrieves covariance matrix from estimation$sigma according to the position (i.e. the state) of the system
    
    for (k in 1:(Samp-spec$lag)){
      for (position in 1:spec$states){
        Eta[k, position] <-  (2*pi)^(-K/2)*det(get_sigma(Sigma, K, position))^-0.5*exp(-0.5*t(u_est[k,])%*%solve(get_sigma(Sigma,K,position))%*%u_est[k,])
      } # end of position loop
    } # end k loop through obs
    
    
    # Filtering probabilities <- first observation is given above as 0.5 for both states
    # see Krolzig (1997) pp. 80 ff.
    for (m in 2:(Samp-spec$lag+1)){
      Ksi[m, 1:spec$states] <- t( Eta[m-1,]*(P%*%Ksi[(m-1),(1:spec$state)]) /as.numeric( t(spec$const)%*%(Eta[(m-1),]*((P%*%Ksi[(m-1),(1:spec$state)]))) ) )
    } # end of m loop through obs
    
    # avoid label switching, order the last state probabilities s.t. the probability for state i < probability for state i   
    Ksi[Samp-spec$lag,] <-sort(Ksi[Samp-spec$lag,], decreasing=FALSE)
    
    #smooting probabilities (inverse time!), refers to Ksi_(t|Samp), i.e. probabilities Ksi given information up to the end of the sample
    # see Krolzig(1997) pp. 84 ff.
    KsiRevTime <-  matrix(NA, (Samp-spec$lag+1),spec$states)
    KsiRevTime[1,] <-  Ksi[Samp-spec$lag+1,]
    
    counter <-  Samp - spec$lag+1
    for (n in 1:(Samp-spec$lag)){
      counter <-  counter-1
      KsiRevTime[n+1, 1:spec$states] <- t( ( t(P)%*% (KsiRevTime[n,1:spec$states] / (P%*%Ksi[counter,(1:spec$states)]))) *Ksi[counter,1:spec$states])
    } # end of n loop through observations
    
    # Flip the matrix to obtain timing 0,1,2,...,Samp-1
    KsiT <- rotate(KsiRevTime)
    # Calculate the sum of probabilities
    Tm <- t(as.matrix(colSums(KsiT)- KsiT[1,]))
    # For estimation of P-Matrix
    KsiT2 <- matrix(NA, (Samp-spec$lag), spec$states^2)
    for (o in 1:(Samp-spec$lag)){
      KsiT2[o,(1:(spec$states^2))] <-  c(t(P))* kronecker(KsiT[o+1,1:spec$states]/(P%*%Ksi[o,1:(spec$states)]) , Ksi[o,(1:spec$states)])
    }# end of o loop through observations 
    # Compute the value of likelihood function (is done for the previous iteration) and the change in the log-likelihood
    LogL <- 0
    for (t in 1:(Samp-spec$lag)){
      LogL <- LogL + log( t(P %*% Ksi[t,]) %*% Eta[t,])
    } # end of t loop throgh observations
    
    add <-cbind(ROUND, LogL,( LogL - LogValue[ROUND,2]) / LogValue[ROUND,2] )
    LogValue <-  rbind(LogValue, add)
    rm(add)
    
    ##########################
    #   Maximization Step    #
    ##########################
    
    #  Estimate the Markov matrix, the first inpup in the reshape() function is the vectorized Markov matrix
    P = t(matrix( colSums(KsiT2) / kronecker( spec$const , kronecker(t(spec$const), diag(spec$states)) %*% colSums(KsiT2) ),spec$states, spec$states))
    
    # Estimate the likelihood function and retrieve matrices B and Lambdas, BL stores B first, then it has diagonal entries of lambdas
    val <-  matrix(0, 1,2)
    val[1,1] <- 1
    val[1,2] <- 1
    RND <- 1
    while ((val[RND,2] > 0.000000001) & (RND<100) ){
      u_est <- residuals4(y, Theta[,RND_ALL],Z, K, spec)
      out_min <- minimize_ms(KsiT,Samp,K,Tm, Lambda, B, spec, u_est ,Theta)
      val_help <- abs( (out_min$val - val[RND,1])/val[RND,1] )
      val_help2 <- cbind(out_min$val, val_help)
      val <- rbind(val, val_help2)
      rm(val_help,val_help2)
      
      # change the position of the instrumented variable according to the new sorting of Lambda
      # spec$pos_instr <- rank(out_min$BL[(K^2+1):( K^2+K)])[spec$pos_instr]
      
      # Store B and Lambda
      out_min$BL <- matrix(out_min$BL,K, (K+spec$states-1))
      # store B (ordered)
      B <-  out_min$BL[,(1:K)]
      beta <- out_min$BL[K,1:(K-1)]
      # store lambda
      for (position in 2:spec$states){
        Lambda[(1+K*(position-1)):(K*position),] <- diag(out_min$BL[,(K+position-1)],K,K)
      }
      
      # Calculate the covariance matrices Sigma_m (for all different Lambda_m) from decomposition of B and lambda
      eigen_contr <- c() # if eigenvalues of Sigma smaller than threshold, break the loop and return with no result
      for (position in 1:spec$states){
        Sigma[(1+K*(position-1)):(K*position),] <- B%*%  Lambda[(1+K*(position-1)):(K*position),]%*%t(B)
        eigen_contr <-  c(eigen_contr, eigen(Sigma[(1+K*(position-1)):(K*position),])$values)
      }
      
      # Estimate parameter vector Theta (add up over the different states, see Appendix (page 21) Herwartz/Lütkepohl 2011 WP)
      
      T2 <- matrix(0, dim(R)[2], dim(R)[2])
      T3 <- matrix(0, dim(R)[2],(Samp-spec$lag)*K)
      
      for (position in 1:spec$states){
        T2 <- T2 + t(R)%*%kronecker( (t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]) %*%Z),solve(get_sigma(Sigma,K,position)))%*%R
        T3 <- T3 + t(R)%*%kronecker(t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]),solve(get_sigma(Sigma,K,position)))
      }
      # Theta <- solve(T2)%*%T3%*%c(y_reg)
      gamma_help <- solve(T2)%*%T3%*%c(z)
      Theta_help <- R%*%gamma_help
      Theta <- cbind(Theta, Theta_help )
      rm(Theta_help)
      
      RND <- RND +1
      RND_ALL <- RND_ALL +1
    } # end of second while loop
    # update initial probability
    # estimate Ksi0,0 = Ksi0,Samp
    Ksi[1,] <- KsiT[1,]
    if (verbose==TRUE){
      cat(paste("Round iteration of EM: ",ROUND, " with LL ", LogL, ". Tol. value ",LogValue[ROUND,3], ", but must become: ", Tolvalue, ".\r\n", sep=""),  file=log_path, append=TRUE)
    }
    ROUND <- ROUND+1
  }# end of first while loop
  ####################################
  if (ROUND <= 500){
    convergence <- 1
  } else {
    convergence <- 0
  }
  # order the Lambdas by size of the lambda_i,j of the first state
  order <- order(out_min$B[(K^2+1):( K^2+K)], decreasing = FALSE)
  # store lambda
  for (position in 2:spec$states){
    Lambda[(1+K*(position-1)):(K*position),] <- diag(diag(Lambda[(1+K*(position-1)):(K*position),])[order] ,K,K)
  }
  B <-  B[,order]
  # extract Theta from matrix of Thetas (ROUND*RND = RND_ALL)
  ThetaLast <- matrix(Theta[,RND],K)
  # num_param <- dim(out_min$BL)[1]*dim(out_min$BL)[2] + dim(ThetaLast)[1]*dim(ThetaLast)[2]
  IterationLik <- val[RND,1]
  
  # order column of zeros last and monetary policy first
  if (spec$restric == "h-var-res" ||  spec$restric == "ph-var" ||  spec$restric == "h-var" || spec$restric == "invarB-res"){
    col_last <- which.min(abs(apply(B, 2, function(x) sum(x[1:(K-1)]))))
    cols <- c(1:K)[-col_last ]
    order <- c(cols, col_last)
    B <- B[,order]
    lambda_vec <- c()
    for (position in 2:spec$states){
      Lambda[(1+K*(position-1)):(K*position),] <- diag(diag(Lambda[(1+K*(position-1)):(K*position),])[order] ,K,K)
      lambda_vec <- c(lambda_vec,diag(Lambda[(1+K*(position-1)):(K*position),]))
    }
    
    # order column of instrument first
    col_ins <- which(abs(B[K,1:(K-1)])==max(abs(B[K,1:(K-1)])), arr.ind = TRUE)
    cols <- c((1:(K-1)))[-col_ins]
    order <- c(col_ins, cols, K)
    B <- B[,order]
    lambda_vec <- c()
    for (position in 2:spec$states){
      Lambda[(1+K*(position-1)):(K*position),] <- diag(diag(Lambda[(1+K*(position-1)):(K*position),])[order] ,K,K)
      lambda_vec <- c(lambda_vec,diag(Lambda[(1+K*(position-1)):(K*position),]))
    }
  }
  
  # Normalize B-matrix to have positive mian diagonal
  for (i in 1:K){
    if (B[i,i] < 0){
      B[,i] <-  -1*B[,i]
    }
  }
  # Normalize B-matrix to have positive monetary policy shock
  if (B[2,1] < 0){
    B[,1] <-  -1*B[,1]
  }
  
  # Impulse responses
  imp_rep <- calc_imp_response(B, ThetaLast, Lambda, spec$horizon, K, spec, spec$lag)
  
  out <- list(Theta =ThetaLast, Sigma = Sigma,KsiT=KsiT, P =P, logL=IterationLik, B=B, Lambda=Lambda, Ksi=Ksi, convergence =convergence, imp_rep =imp_rep$impulse, imp_rep_cum = imp_rep$impulse_cum, Phi_coef = imp_rep$Phi)
  return(out)
  
}
