estimate_ms_unres <- function(A1, A2, Samp,K, y, Z, spec, Tolvalue, RandNumbers,code, log_path, verbose,start_val){
  y_reg <- c(t(y)) # transpose and vectorize data 
  # See in paper of Herwartz and Lütkepohl (2011) pp 19ff. an explanation of the algorithm.  
  #--------------------------------------------------------------------------------------------
  ################
  # EM Algorithm #      
  ################
  #--------------------------------------------------------------------------------------------
  ##########################
  # Set initial parameters #
  ##########################
  # Counter that accounts for number of iterations (ROUND (first while loop)) and number of iterations within each round (RND (second while loop))
  RND_ALL <- 1
  verbose =TRUE
  # initial tranistion matrix
  P <-  1/spec$states*spec$const%*%t(spec$const)
  if (!is.null(start_val$P)){
    P <- start_val$P
  }
  # initial coefficient vector
  Theta <-  solve(A1)%*%A2
  if (!is.null(start_val$Theta)){
    Theta <-as.matrix(c(start_val$Theta))
  }
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, K, K*spec$lag+1)
  
  u_est <- residuals3(y,Theta_mat,Z)  
  
  # initial KSI 
  Ksi <- t(spec$const/spec$states)  
  if (!is.null(start_val$Ksi)){
    Ksi <-as.matrix(start_val$Ksi[1,])
  }
  
  sigma <- (Samp-spec$lag)^-1*(t(u_est)%*%u_est)
  
  # Form initial covariance matrices
  Sigma <- matrix(NA,K*spec$states,K)
  for (position in  1:spec$states){
    Sigma[((1+K*(position-1)):(K*position)), ]  <- sigma*position^0.3
  }
  
  if (!is.null(start_val$Sigma)){
    Sigma <- start_val$Sigma + matrix(rep(RandNumbers,spec$states), spec$states*K,K) 
  }
  
  # Initialize the likelihood values
  LogValue  <- matrix(NA, 1,3)
  LogValue[1,1] <- 0
  LogValue[1,2] <- 1
  LogValue[1,3] <- 1
  
  # Round is a counter to accumulate number of iterations 
  # Estimate until convergence is achieved
  ROUND <- 1
  
  while((abs(LogValue[ROUND,3]) > Tolvalue)  & (ROUND<=500)){
    
    ##########################
    #     Expectation step   #
    ##########################
    
    # Calculate the conditional distribution function Eta
    # In the first round Eta needs to be created 
    # In the next rounds Eta exists and only needs to be updated.
    # Function get_sigma retrieves covariance matrix from estimation$sigma according to the position (i.e. the state) of the system
    if (ROUND == 1){
      Eta <- matrix(NA,(Samp-spec$lag),spec$states)
    }
    for (k in 1:(Samp-spec$lag)){
      for (position in 1:spec$states){
        
        Eta[k, position] <-  (2*pi)^(-K/2)*det(get_sigma(Sigma, K, position))^-0.5*exp(-0.5*t(u_est[k,])%*%solve(get_sigma(Sigma,K,position))%*%u_est[k,])
      } # end of position loop
      
    } # end k loop through obs
    
    # Filtering probabilities <- first observation is given above as 0.5 for both states
    # see Krolzig (1997) pp. 80 ff.
    if (ROUND == 1){
      Ksi <-  matrix(NA, Samp-spec$lag+1, spec$states)
      # initial KSI
      Ksi[1,] <-  t(spec$const/spec$states)   
    }      
    for (m in 2:(Samp-spec$lag+1)){
      Ksi[m, 1:spec$states] <- t( Eta[m-1,]*(P%*%Ksi[(m-1),(1:spec$state)]) /as.numeric( t(spec$const)%*%(Eta[(m-1),]*((P%*%Ksi[(m-1),(1:spec$state)]))) ) )
    } # end of m loop through obs

    # avoid label switching, order the last state probabilities s.t. the probability for state i < probability for state i   
    Ksi[Samp-spec$lag,] <-sort(Ksi[Samp-spec$lag,], decreasing=FALSE)
    
    #smooting probabilities (inverse time!), refers to Ksi_(t|Samp), i.e. probabilities Ksi given information up to the end of the sample
    # see Krolzig(1997) pp. 84 ff.
    KsiRevTime <-  matrix(NA, (Samp-spec$lag+1),spec$states)
    KsiRevTime[1,] <-  Ksi[Samp-spec$lag+1,]
    
    counter <-  Samp - spec$lag+1
    for (n in 1:(Samp-spec$lag)){
      counter <-  counter-1
      KsiRevTime[n+1, 1:spec$states] <- t( ( t(P)%*% (KsiRevTime[n,1:spec$states] / (P%*%Ksi[counter,(1:spec$states)]))) *Ksi[counter,1:spec$states])
    } # end of n loop through observations
    
    # Flip the matrix to obtain timing 0,1,2,...,Samp-1
    KsiT <- rotate(KsiRevTime)
    # Calculate the sum of probabilities
    Tm <- t(as.matrix(colSums(KsiT)- KsiT[1,]))
    # For estimation of P-Matrix
    KsiT2 <- matrix(NA, (Samp-spec$lag), spec$states^2)
    for (o in 1:(Samp-spec$lag)){
      KsiT2[o,(1:(spec$states^2))] <-  c(t(P))* kronecker(KsiT[o+1,1:spec$states]/(P%*%Ksi[o,1:(spec$states)]) , Ksi[o,(1:spec$states)])
    }# end of o loop through observations 
    # Compute the value of likelihood function (is done for the previous iteration) and the change in the log-likelihood
    LogL <- 0
    for (t in 1:(Samp-spec$lag)){
      LogL <- LogL + log( t(P %*% Ksi[t,]) %*% Eta[t,])
    } # end of t loop throgh observations
    
    add <-cbind(ROUND, LogL,( LogL - LogValue[ROUND,2]) / LogValue[ROUND,2] )
    LogValue <-  rbind(LogValue, add)
    rm(add)
    
    ##########################
    #   Maximization Step    #
    ##########################
    
    #  Estimate the Markov matrix, the first inpup in the reshape() function is the vectorized Markov matrix
    P = t(matrix( colSums(KsiT2) / kronecker( spec$const , kronecker(t(spec$const), diag(spec$states)) %*% colSums(KsiT2) ),spec$states, spec$states))
    
    # Estimate the likelihood function and retrieve matrices B and Lambdas, BL stores B first, then it has diagonal entries of lambdas
    val <-  matrix(0, 1,2)
    val[1,1] <- 1
    val[1,2] <- 1
    RND <- 1
    # res_proxy <- res_proxy[2,]
    # start_time <- proc.time()
    while ((val[RND,2] > 0.000000001) & (RND<100) ){
      u_est <- residuals4(y, Theta[,RND_ALL],Z, K, spec)
      out_min <- minimize_ms_unres(KsiT,Samp,K,Tm,Sigma, spec, u_est, Theta, RND_ALL)
      
      
      val_help <- abs( (out_min$val - val[RND,1])/val[RND,1] )
      val_help2 <- cbind(out_min$val, val_help)
      val <- rbind(val, val_help2)
      rm(val_help,val_help2)
      
      # Calculate the covariance matrices Sigma from vector of estimates
      Sigma <- c()
      eigen_contr <-  c()
      breaks <- c(seq(1,((K*(K+1)/2)*spec$states), (K*(K+1)/2)),(K*(K+1)/2)*spec$states+1)
      for (j in 1:(length(breaks)-1)){
        Sigma_help <- symMatrix(out_min$Sigma[breaks[j]:(breaks[(j+1)]-1)])
        # eigen_contr <-  c(eigen_contr, eigen(Sigma_help)$values)
        Sigma <- rbind(Sigma, Sigma_help)
      }
      # Calculate the covariance matrices Sigma_m (for all different Lambda_m) from decomposition of B and lambda
      eigen_contr <- c() # if eigenvalues of Sigma smaller than threshold, break the loop and return with no result
      for (position in 1:spec$states){
             eigen_contr <-  c(eigen_contr, eigen(Sigma[(1+K*(position-1)):(K*position),])$values)
      }
      # Estimate parameter vector Theta (add up over the different states, see Appendix (page 21) Herwartz/Lütkepohl 2011 WP)
      T2 <- matrix(0, (K*spec$lag+1)*K, (K*spec$lag+1)*K)
      T3 <- matrix(0, (K*spec$lag+1)*K,(Samp-spec$lag)*K)
      for (position in 1:spec$states){
        T2 <- T2 + kronecker( (t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]) %*%Z),solve(get_sigma(Sigma,K,position)))
        T3 <- T3 + kronecker(t(Z)%*%diag(KsiT[2:(Samp-spec$lag+1),position]),solve(get_sigma(Sigma,K,position)))
      }
      Theta_help <- solve(T2)%*%T3%*%c(y_reg)
      
      Theta <- cbind(Theta, Theta_help )
      rm(Theta_help)
      
      RND <- RND +1
      RND_ALL <- RND_ALL +1
    } # end of second while loop
    #update initial probability
    # estimate Ksi0,0 = Ksi0,Samp
    Ksi[1,] <- KsiT[1,]
    if (verbose==TRUE){
      cat(paste("Round iteration of EM: ",ROUND, " with LL ", LogL, ". Tol. value ",LogValue[ROUND,3], ", but must become: ", Tolvalue, ".\r\n", sep=""),  file=log_path, append=TRUE)
    }
    
    ROUND <- ROUND+1
  }# end of first while loop
  ####################################
  if (ROUND <= 500){
    convergence <- 1
  } else if (ROUND > 500){
    convergence <- 0
  }
  # get value for xi
  
  # extract Theta from matrix of Thetas (ROUND*RND = RND_ALL)
  ThetaLast <- matrix(Theta[,RND_ALL],K, 1+spec$lag*K)
  u_est <- residuals4(y, ThetaLast,Z, K, spec)
  # update the value of the likelihood for the last iteration
  Eta2 <- matrix(NA, (Samp-spec$lag), spec$states)
  for (j in 1:(Samp-spec$lag)){
    for (position in 1:spec$states){
      Eta2[j,position] <- (2*pi)^(-K/2)*det(get_sigma(Sigma,K,position))^-0.5*exp(-0.5*t(u_est[j,])%*%solve(get_sigma(Sigma,K,position))%*%u_est[j,] )
    } # end of loop through position
  } #end of loop through j
  
  #update state probabilities for the changes in parameter values
  for (m in 2:(Samp-spec$lag +1)){
    Ksi[m,1:spec$states] <-t( Eta2[(m-1),]*(P%*%Ksi[(m-1),1:spec$states])/as.numeric(t(spec$const)%*%(Eta2[m-1,]*(P%*%Ksi[(m-1),1:spec$states]))))
  } # end of loop through m
  
  # calculate the approximated derivative for each t and each parameter
  logLikeVec2 <- matrix(NA, (Samp-spec$lag),1)
  for (t in 1:(Samp-spec$lag)){
    logLikeVec2[t,1] <- log( t(P%*%Ksi[t,])%*% Eta2[t,] )
  } # end of loop through t
  
  LogValue[ROUND,2] <- sum(logLikeVec2[,1])
  
  IterationLik <- rbind(LogValue[ROUND,2], val[RND,1])
  out <- list(Theta =ThetaLast, Sigma = Sigma, u_est = u_est, KsiT=KsiT, P =P, logL=IterationLik, xi=out_min$xi, Eta=Eta2, Ksi=Ksi, convergence =convergence)
  return(out)
} # end of function
