ms_var <- function(spec, y, Z, LM, Samp, K, y_start, log_path, code, date_list, verbose, start_val=c(),aug_var, b_method="rnd"){
  est_ms <- vector(mode = "list", length = spec$iterations)
  Tolvalue <-  10^-9
  y_reg <- c(t(y)) # transpose and vectorize data 
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  # matrix of random numbers+
  RandNumbers <- matrix(NA,K,K*spec$iterations)
  for (iteration in 1:spec$iterations){
    RandNumbers[,(1+K*(iteration-1)):(K*iteration)] <- 1/1000*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K)
  }
  if (!spec$unres){
    for (iteration in 1:spec$iterations) {
      if (spec$sim == !TRUE){
        cat(paste("MSwitch: Iteration: ", iteration, " of model. Total: ", spec$iterations ,"\r\n"),  file=log_path, append=TRUE)
      }
      if (aug_var){ # if there is an instrument in the model, automatically restrict its impact by Restricted OLS
        if (spec$clean == FALSE){
          est_ms_it <-  tryCatch({estimate_ms_res(A1,A2,Samp,K,y, Z,spec, Tolvalue, RandNumbers=RandNumbers[,(1+K*(iteration-1)):(K*iteration)], LM=LM[iteration], code, log_path, iteration, verbose=verbose, start_val,b_method=b_method)}, error = function(e) NULL)
        
          } else if (spec$clean == TRUE){
          est_ms_it <-  estimate_ms(A1,A2,Samp,K,y, Z,spec, Tolvalue, RandNumbers[,(1+K*(iteration-1)):(K*iteration)], LM[iteration], code, log_path, iteration, verbose=verbose, start_val=start_val,b_method=b_method)
        } 
      } else if (!aug_var){ # this is the regular Expectation-Maximization algorithm, if no instrument is applied
        est_ms_it <-  tryCatch({estimate_ms_norm(A1,A2,Samp,K,y, Z,spec, Tolvalue, RandNumbers[,(1+K*(iteration-1)):(K*iteration)], LM[iteration], code, log_path, iteration, verbose=verbose,b_method=b_method)}, error = function(e) NULL)
      }
      if (!is.null(est_ms_it)){
        se <- tryCatch({param_stderr(Samp, K, estimation = est_ms_it, spec, y, Z, log_path, way = 1, amod=FALSE)}, error = function(e) NULL)
        # Calculate the standardized residuals
        u_stand <- tryCatch({u_stand_ms(Samp,K,y,Z,spec,est_ms_it)}, error = function(e) NULL)
      } else {
        se <- NULL
        se_lambda <- NULL
        u_stand <- NULL
      }
      if (!is.null(est_ms_it) & !is.null(se) & !is.null(u_stand)){ 
        attr(est_ms_it$logL, "dimnames") <- NULL
        # append list of results
        est_ms_it$hessian <-  se$hessian
        est_ms_it$param <-  se$param
        # wald test
        # est_ms_it$wald_lambda <- wald_test_lambda(est_ms_it, Samp, K, spec)
        
        # Calculate the residuals
        est_ms_it$u_est <- residuals3(y,est_ms_it$Theta,Z)
        
        # Heteroskedasticity Tests --> H0 is no conditional heteroskedasticity
        Q1 <- uni_cov(u_stand$u_st, Samp, K,spec, 10)
        Q2 <- multi_cov(u_stand$u_st,K ,Samp, spec, 10, 0)
        LM_test <- LM_regression(u_stand$u_st,K ,Samp, spec, 10, 0)
        # Portmanteau test for correlation in residuals
        portmanteau <- portmanteau_func(est_ms_it$u_est,K, Samp, spec, h=16)
        # Lm test for residual autocorrelation
        resid_corr <- list()
        resid_corr$p_LM_resid <- "not available"
        # num_param = K + spec$lag*K^2 + spec$states^2 + K*(K+1)/2
        # int + p*coef_mat + P + B + lambda
        # to account for trend or no trend
        add <- 1
        if (dim(est_ms_it$Theta)[2]/(K*spec$lag+1)!=1){
          add <- 2
        }
        
        num_param <-  K*add + spec$lag*K^2 + (spec$states-1)*(spec$states)  + (K^2) +K*(spec$states-1) 
        # selection criteria
        
        selec_crit <- selec_crit_calc(est_ms_it$logL[1],K ,Samp,spec$lag, num_param)
        for (i in 1:spec$states){
          Sigma1 <- est_ms_it$Sigma[1:K,1:K]
          Sigma2 <- est_ms_it$Sigma[(K+1):(2*K),1:K]
          if (spec$states > 2){
            Sigma3 <- est_ms_it$Sigma[(K*2+1):(3*K),1:K]
          } else if (spec$states >3){
            Sigma4 <- est_ms_it$Sigma[(K*3+1):(4*K),1:K]
          }
        }
# Summarize results in list        
        est_ms[[iteration]] <- list(se_struc = est_ms_it$se_struc,beta=est_ms_it$beta, eta=est_ms_it$Eta, Theta =est_ms_it$Theta,  Sigma = est_ms_it$Sigma, ll_ms = est_ms_it$logL[1], 
                                    u_est = est_ms_it$u_est,  u_stand= u_stand$u_st, Sigma_t = u_stand$Sigma_t, AIC=selec_crit$AIC,HQ=selec_crit$HQ,SC=selec_crit$SC, 
                                    u_hettest = Q1$p_Q1, m_hettest = Q2$p_Q2, LM_test = LM_test$p_LM, num_param = num_param,stand_err = se$Sterrors, imp_rep = est_ms_it$imp_rep, 
                                    imp_rep_cum = est_ms_it$imp_rep_cum, Phi_coef =est_ms_it$Phi_coef, KsiT= est_ms_it$KsiT, Lambda = est_ms_it$Lambda,B=est_ms_it$B, P= est_ms_it$P, 
                                    hist_decomp = est_ms_it$hist_decomp, hist_decomp_contrib = est_ms_it$hist_decomp_contrib, struc_shocks = est_ms_it$struc_shocks, st_struc_shocks= est_ms_it$st_struc_shocks,
                                    process_mean = est_ms_it$process_mean, imp_rep = est_ms_it$imp_rep, imp_rep_cum = est_ms_it$imp_rep_cum, variance_decomp =est_ms_it$variance_decomp, 
                                    port_p =portmanteau$port_p, port_p_mod =portmanteau$port_p_mod,  p_LM_resid_corr =  resid_corr$p_LM_resid, B_se=se$B_se, Lambda_se = se$Lambda_se, 
                                    Var_se = se$Var_se,P_se =se$P_se, convergence=est_ms_it$convergence)
        
      } else { # test for is.null
        est_ms_it$ll_ms = -Inf
        est_ms[[iteration]] <- list(ll_ms = est_ms_it$ll_ms) 
      }
    } #end of iterations loop
    # check for the highest likelihood of all iterations
    Lik <- c()
    for (i in 1:spec$iterations){
      Lik[i] <-   est_ms[[i]]$ll_ms
    }
    iteration_mx <- which.max(Lik)
    no_nonconv <- is.infinite(Lik)
    no_nonconv  <-   length(no_nonconv[no_nonconv==TRUE])
    models_con <- spec$iterations - no_nonconv
    
    if (no_nonconv == spec$iterations){
      return("No convergence. Check model.")
    } else {
      convergence_all <- paste0("No of models estimated: ",spec$iterations, ". Converged models: ",models_con ,".")
    }
    ########## BOOTSTRAPPING
    if (spec$boot =="yes"){
      cat(paste("MSwitch: Starting bootstrap... Check Log-File for Progress! \r\n"))
      cat(paste("MSwitch: Starting bootstrap\r\n"),  file=log_path, append=TRUE)
      cat(paste("To check existence, this is the iteration max: ", LM[iteration_mx],"\r\n"),  file=log_path, append=TRUE)
      cat(paste("This is the LL max: ", est_ms[[iteration_mx]]$ll_ms,"\r\n"),  file=log_path, append=TRUE)
      
      boot_int <- bootstrap_ms(est_ms[[iteration_mx]], spec$lag, spec, Samp, K, y, y_start, LM[iteration_mx],log_path, verbose)
      # Summarize results in list      
      estimation <- list(se_struc = est_ms[[iteration_mx]]$se_struc, beta=est_ms[[iteration_mx]]$beta,convergence_all = convergence_all, highest_ll=iteration_mx,  ll_all = Lik, Theta =est_ms[[iteration_mx]]$Theta, Sigma = est_ms[[iteration_mx]]$Sigma, ll_ms = est_ms[[iteration_mx]]$ll_ms, 
                         u_est = est_ms[[iteration_mx]]$u_est,  u_stand= est_ms[[iteration_mx]]$u_stand, Sigma_t = est_ms[[iteration_mx]]$Sigma_t, AIC=est_ms[[iteration_mx]]$AIC,
                         HQ=est_ms[[iteration_mx]]$HQ,SC=est_ms[[iteration_mx]]$SC, u_hettest = est_ms[[iteration_mx]]$u_hettest, m_hettest = est_ms[[iteration_mx]]$m_hettest, 
                         LM_test = est_ms[[iteration_mx]]$LM_test, num_param = est_ms[[iteration_mx]]$num_param, stand_err = est_ms[[iteration_mx]]$stand_err, 
                         imp_rep = est_ms[[iteration_mx]]$imp_rep, imp_rep_cum = est_ms[[iteration_mx]]$imp_rep_cum, Phi_coef =est_ms[[iteration_mx]]$Phi_coef, 
                         KsiT= est_ms[[iteration_mx]]$KsiT, Lambda = est_ms[[iteration_mx]]$Lambda,B=est_ms[[iteration_mx]]$B, P= est_ms[[iteration_mx]]$P, 
                         hist_decomp = est_ms[[iteration_mx]]$hist_decomp, hist_decomp_contrib = est_ms[[iteration_mx]]$hist_decomp_contrib, 
                         struc_shocks = est_ms[[iteration_mx]]$struc_shocks, st_struc_shocks= est_ms[[iteration_mx]]$st_struc_shocks, 
                         process_mean = est_ms[[iteration_mx]]$process_mean, variance_decomp =est_ms[[iteration_mx]]$variance_decomp, port_p =est_ms[[iteration_mx]]$port_p, 
                         port_p_mod =est_ms[[iteration_mx]]$port_p_mod,  p_LM_resid_corr =  est_ms[[iteration_mx]]$p_LM_resid_corr,B_se=est_ms[[iteration_mx]]$B_se, 
                         Lambda_se = est_ms[[iteration_mx]]$Lambda_se, Var_se = est_ms[[iteration_mx]]$Var_se,P_se =est_ms[[iteration_mx]]$P_se,convergence=est_ms[[iteration_mx]]$convergence,  
                         upper_int5 = boot_int$upper_int5, lower_int5 = boot_int$lower_int5,  upper_int10 = boot_int$upper_int10, lower_int10 = boot_int$lower_int10, upper_int68 = boot_int$upper_int68, lower_int68 = boot_int$lower_int68,  
                         upper_int5_cum = boot_int$upper_int5_cum, lower_int5_cum = boot_int$lower_int5_cum, upper_int10_cum = boot_int$upper_int10_cum, lower_int10_cum = boot_int$lower_int10_cum, upper_int68_cum = boot_int$upper_int68_cum, lower_int68_cum = boot_int$lower_int68_cum)
      cat(paste("MSwitch: Bootstrap completed. \r\n"))
    }
    if (spec$boot !="yes"){
      # Summarize results in list      
      estimation <- list(se_struc = est_ms[[iteration_mx]]$se_struc, beta=est_ms[[iteration_mx]]$beta,convergence_all = convergence_all, highest_ll=iteration_mx, ll_all = Lik , Theta =est_ms[[iteration_mx]]$Theta, Sigma = est_ms[[iteration_mx]]$Sigma, ll_ms = est_ms[[iteration_mx]]$ll_ms, 
                         u_est = est_ms[[iteration_mx]]$u_est,  u_stand= est_ms[[iteration_mx]]$u_stand, Sigma_t = est_ms[[iteration_mx]]$Sigma_t, AIC=est_ms[[iteration_mx]]$AIC,
                         HQ=est_ms[[iteration_mx]]$HQ,SC=est_ms[[iteration_mx]]$SC, u_hettest = est_ms[[iteration_mx]]$u_hettest, m_hettest = est_ms[[iteration_mx]]$m_hettest, 
                         LM_test = est_ms[[iteration_mx]]$LM_test, num_param = est_ms[[iteration_mx]]$num_param, stand_err = est_ms[[iteration_mx]]$stand_err, 
                         imp_rep = est_ms[[iteration_mx]]$imp_rep, imp_rep_cum = est_ms[[iteration_mx]]$imp_rep_cum, Phi_coef =est_ms[[iteration_mx]]$Phi_coef, 
                         KsiT= est_ms[[iteration_mx]]$KsiT, Lambda = est_ms[[iteration_mx]]$Lambda,B=est_ms[[iteration_mx]]$B, P= est_ms[[iteration_mx]]$P, 
                         hist_decomp = est_ms[[iteration_mx]]$hist_decomp, hist_decomp_contrib = est_ms[[iteration_mx]]$hist_decomp_contrib, 
                         struc_shocks = est_ms[[iteration_mx]]$struc_shocks, st_struc_shocks= est_ms[[iteration_mx]]$st_struc_shocks, process_mean = est_ms[[iteration_mx]]$process_mean,
                         variance_decomp =est_ms[[iteration_mx]]$variance_decomp, port_p =est_ms[[iteration_mx]]$port_p, port_p_mod =est_ms[[iteration_mx]]$port_p_mod,  
                         p_LM_resid_corr =  est_ms[[iteration_mx]]$p_LM_resid_corr, B_se=est_ms[[iteration_mx]]$B_se, Lambda_se = est_ms[[iteration_mx]]$Lambda_se, 
                         Var_se = est_ms[[iteration_mx]]$Var_se,P_se =est_ms[[iteration_mx]]$P_se, convergence=est_ms[[iteration_mx]]$convergence)
    }
if (spec$restric == "ph-var"){
     cat(paste("Finished iteration",spec$i,"\r\n"),  file=log_path, append=TRUE)
 }
    return(estimation)
  } else if (spec$unres){
    # no Structure on covariance matrix --> reduced form estimation only
    RandNumbers<- 1/1000*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K)
    
    est_ms <- estimate_ms_unres(A1,A2,Samp,K,y,Z,spec, Tolvalue, RandNumbers, code, log_path, verbose, start_val)
    est_ms$num_param <-  K + spec$lag*K^2 + (spec$states-1)*(spec$states)  + K*(K+1)/2*(spec$states) 
    est_ms$selec_crit <- selec_crit_calc(est_ms$logL[1],K ,Samp,spec$lag, est_ms$num_param)
    return(est_ms)  
  }
  
} # end of function

