ms_var_ins <- function(spec, y, Z, LM, Samp, K, y_start, log_path, code, date_list, verbose, start_val,var_aug=FALSE){
  
  est_ms <- vector(mode = "list", length = spec$iterations)
  Tolvalue <-  10^-9
  y_reg <- c(t(y)) # transpose and vectorize data 
  
  A1 <- kronecker(t(Z),diag(dim(y)[2]))%*%kronecker(Z,diag(dim(y)[2]))
  A2 <- kronecker(t(Z), diag(dim(y)[2]))%*%c(y)
  
  # matrix of random numbers
  RandNumbers <- matrix(NA,K,K*spec$iterations)
  for (iteration in 1:spec$iterations){
    RandNumbers[,(1+K*(iteration-1)):(K*iteration)] <- 1/1000*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K)
  }
  # start <- proc.time()
  if (!spec$unres){
    for (iteration in 1:spec$iterations) {
      est_ms_it <- estimate_ms_ins(A1,A2,Samp,K,y, Z,spec, Tolvalue, RandNumbers[,(1+K*(iteration-1)):(K*iteration)], LM[iteration], code, log_path, iteration, verbose=TRUE, start_val=c(),var_aug=var_aug)
      if (!is.null(est_ms_it)){ 
        attr(est_ms_it$logL, "dimnames") <- NULL
        
        # Calculate standard errors
        if (var_aug == TRUE){
          se <- param_stderr_ins(Samp, K, estimation = est_ms_it, spec, y, Z, log_path, way = 1)
        } else if(var_aug==FALSE){
          se <- param_stderr(Samp, K, estimation = est_ms_it, spec, y, Z, log_path, way = 1)
        }
        # append list of results
        # Calculate the residuals
        est_ms_it$u_est <- residuals3(y,est_ms_it$Theta,Z)
        # Calculate the standardized residuals
        u_stand <- u_stand_ms_aug(Samp,K,y,Z,spec,est_ms_it)
        # H0 is no conditional heteroskedasticity
        Q1 <- uni_cov(u_stand$u_st, Samp, K,spec, 10)
        Q2 <- multi_cov(u_stand$u_st,K ,Samp, spec, 10, 0)
        LM_test <- LM_regression(u_stand$u_st,K ,Samp, spec, 10, 0)
        
        # Portmanteau test for correlation in residuals
        if (var_aug == TRUE){
          portmanteau <- portmanteau(est_ms_it$u_est[,1:K],K, Samp, spec, h=16)
          resid_corr <- LM_test_res_corr(est_ms_it$u_est[,1:K], Z, K, Samp,spec,  h=16)
          num_param <-  K + spec$lag*dim(y)[2]^2 + (spec$states-1)*(spec$states)  + (K^2) +K*(spec$states-1) + length(beta) + length(lambda_nu)
        } else if(var_aug==FALSE){
          portmanteau <- portmanteau(est_ms_it$u_est,K, Samp, spec, h=16)
          resid_corr <- LM_test_res_corr(est_ms_it$u_est, Z, K, Samp,spec,  h=16)
          num_param <-  K + spec$lag*K^2 + (spec$states-1)*(spec$states)  + (K^2) +K*(spec$states-1)
        }
        # selection criteria
        
        selec_crit <- selec_crit_calc(est_ms_it$logL[1],K ,Samp,spec$lag, num_param)
        for (i in 1:spec$states){
          Sigma1 <- est_ms_it$Sigma[1:K,1:K]
          Sigma2 <- est_ms_it$Sigma[(K+1):(2*K),1:K]
          if (spec$states > 2){
            Sigma3 <- est_ms_it$Sigma[(K*2+1):(3*K),1:K]
          } else if (spec$states >3){
            Sigma4 <- est_ms_it$Sigma[(K*3+1):(4*K),1:K]
          }
        }
        
        est_ms[[iteration]] <- list(eta=est_ms_it$Eta, Theta =est_ms_it$Theta,  Sigma = est_ms_it$Sigma, ll_ms = est_ms_it$logL[1],
                                    u_est = est_ms_it$u_est,  u_stand= u_stand$u_st, Sigma_t = u_stand$Sigma_t, AIC=selec_crit$AIC,HQ=selec_crit$HQ,SC=selec_crit$SC, 
                                    u_hettest = Q1$p_Q1, m_hettest = Q2$p_Q2, LM_test = LM_test$p_LM, num_param = num_param,stand_err = se$Sterrors, imp_rep = est_ms_it$imp_rep, 
                                    imp_rep_cum = est_ms_it$imp_rep_cum, Phi_coef =est_ms_it$Phi_coef, KsiT= est_ms_it$KsiT, Lambda = est_ms_it$Lambda,B=est_ms_it$B, P= est_ms_it$P, 
                                    hist_decomp = est_ms_it$hist_decomp, hist_decomp_contrib = est_ms_it$hist_decomp_contrib, struc_shocks = est_ms_it$struc_shocks, st_struc_shocks= est_ms_it$st_struc_shocks,
                                    process_mean = est_ms_it$process_mean, imp_rep = est_ms_it$imp_rep, imp_rep_cum = est_ms_it$imp_rep_cum, variance_decomp =est_ms_it$variance_decomp, 
                                    port_p =portmanteau$port_p, port_p_mod =portmanteau$port_p_mod,  p_LM_resid_corr =  resid_corr$p_LM_resid ,B_se = se$B_se, Lambda_se = se$Lambda_se, 
                                    Var_se = se$Var_se,P_se =se$P_se,  beta_se =se$beta_Sterrors, lambda_nu_se =se$lambda_nu_Sterrors, beta =est_ms_it$beta, lambda_nu =est_ms_it$lambda_nu, convergence=est_ms_it$convergence)
        
      } else if (is.null(est_ms_it)){ # test for is.null
        est_ms_it$ll_ms = -Inf
        est_ms[[iteration]] <- list(ll_ms = est_ms_it$ll_ms) 
      }
    } #end of iterations loop
    
    # check for the highest likelihood of all iterations
    Lik <- c()
    for (i in 1:spec$iterations){
      Lik[i] <-   est_ms[[i]]$ll_ms
    }
    iteration_mx <- which.max(Lik)
    no_nonconv <- is.infinite(Lik)
    no_nonconv  <-   length(no_nonconv[no_nonconv==TRUE])
    models_con <- spec$iterations - no_nonconv
    
    if (no_nonconv == spec$iterations){
      return("No convergence. Check model.")
    } else {
      convergence_all <- paste0("No of models estimated: ",spec$iterations, ". Converged models: ",models_con ,".")
    }
    
    # 
    if (spec$boot =="yes"){
      cat(paste("MSwitch: Starting bootstrap... Check Log-File for Progress! \r\n"))
      cat(paste("MSwitch: Starting bootstrap\r\n"),  file=log_path, append=TRUE)
      boot_int <- bootstrap_ms(est_ms[[iteration_mx]], res_proxy, spec$lag, spec, Samp, K, y, y_start, LM[iteration_mx],log_path, verbose)
      
      estimation <- list(convergence_all = convergence_all, highest_ll=iteration_mx,  ll_all = Lik, Theta =est_ms[[iteration_mx]]$Theta, Sigma = est_ms[[iteration_mx]]$Sigma, ll_ms = est_ms[[iteration_mx]]$ll_ms, 
                         u_est = est_ms[[iteration_mx]]$u_est,  u_stand= est_ms[[iteration_mx]]$u_stand, Sigma_t = est_ms[[iteration_mx]]$Sigma_t, AIC=est_ms[[iteration_mx]]$AIC,
                         HQ=est_ms[[iteration_mx]]$HQ,SC=est_ms[[iteration_mx]]$SC, u_hettest = est_ms[[iteration_mx]]$u_hettest, m_hettest = est_ms[[iteration_mx]]$m_hettest, 
                         LM_test = est_ms[[iteration_mx]]$LM_test, num_param = est_ms[[iteration_mx]]$num_param, stand_err = est_ms[[iteration_mx]]$stand_err, 
                         imp_rep = est_ms[[iteration_mx]]$imp_rep, imp_rep_cum = est_ms[[iteration_mx]]$imp_rep_cum, Phi_coef =est_ms[[iteration_mx]]$Phi_coef, 
                         KsiT= est_ms[[iteration_mx]]$KsiT, Lambda = est_ms[[iteration_mx]]$Lambda,B=est_ms[[iteration_mx]]$B, P= est_ms[[iteration_mx]]$P, 
                         hist_decomp = est_ms[[iteration_mx]]$hist_decomp, hist_decomp_contrib = est_ms[[iteration_mx]]$hist_decomp_contrib, 
                         struc_shocks = est_ms[[iteration_mx]]$struc_shocks, st_struc_shocks= est_ms[[iteration_mx]]$st_struc_shocks, 
                         process_mean = est_ms[[iteration_mx]]$process_mean, variance_decomp =est_ms[[iteration_mx]]$variance_decomp, port_p =est_ms[[iteration_mx]]$port_p, 
                         port_p_mod =est_ms[[iteration_mx]]$port_p_mod,  p_LM_resid_corr =  est_ms[[iteration_mx]]$p_LM_resid_corr,B_se = est_ms[[iteration_mx]]$B_se, 
                         Lambda_se = est_ms[[iteration_mx]]$Lambda_se, Var_se = est_ms[[iteration_mx]]$Var_se,P_se =est_ms[[iteration_mx]]$P_se, beta_se =est_ms[[iteration_mx]]$beta_se, 
                         lambda_nu_se =est_ms[[iteration_mx]]$lambda_se, 
                         beta =est_ms[[iteration_mx]]$beta, lambda_nu =est_ms[[iteration_mx]]$lambda_nu,convergence=est_ms[[iteration_mx]]$convergence,  
                         upper_int5 = boot_int$upper_int5, lower_int5 = boot_int$lower_int5, upper_int68 = boot_int$upper_int68, lower_int68 = boot_int$lower_int68,  
                         upper_int5_cum = boot_int$upper_int5_cum, lower_int5_cum = boot_int$lower_int5_cum, upper_int68_cum = boot_int$upper_int68_cum, lower_int68_cum = boot_int$lower_int68_cum)
      cat(paste("MSwitch: Bootstrap completed. \r\n"))
    }
    if (spec$boot !="yes"){
      estimation <- list(convergence_all = convergence_all, highest_ll=iteration_mx, ll_all = Lik , Theta =est_ms[[iteration_mx]]$Theta, Sigma = est_ms[[iteration_mx]]$Sigma, ll_ms = est_ms[[iteration_mx]]$ll_ms, 
                         u_est = est_ms[[iteration_mx]]$u_est,  u_stand= est_ms[[iteration_mx]]$u_stand, Sigma_t = est_ms[[iteration_mx]]$Sigma_t, AIC=est_ms[[iteration_mx]]$AIC,
                         HQ=est_ms[[iteration_mx]]$HQ,SC=est_ms[[iteration_mx]]$SC, u_hettest = est_ms[[iteration_mx]]$u_hettest, m_hettest = est_ms[[iteration_mx]]$m_hettest, 
                         LM_test = est_ms[[iteration_mx]]$LM_test, num_param = est_ms[[iteration_mx]]$num_param, stand_err = est_ms[[iteration_mx]]$stand_err, 
                         imp_rep = est_ms[[iteration_mx]]$imp_rep, imp_rep_cum = est_ms[[iteration_mx]]$imp_rep_cum, Phi_coef =est_ms[[iteration_mx]]$Phi_coef, 
                         KsiT= est_ms[[iteration_mx]]$KsiT, Lambda = est_ms[[iteration_mx]]$Lambda,B=est_ms[[iteration_mx]]$B, P= est_ms[[iteration_mx]]$P, 
                         hist_decomp = est_ms[[iteration_mx]]$hist_decomp, hist_decomp_contrib = est_ms[[iteration_mx]]$hist_decomp_contrib, 
                         struc_shocks = est_ms[[iteration_mx]]$struc_shocks, st_struc_shocks= est_ms[[iteration_mx]]$st_struc_shocks, process_mean = est_ms[[iteration_mx]]$process_mean,
                         variance_decomp =est_ms[[iteration_mx]]$variance_decomp, port_p =est_ms[[iteration_mx]]$port_p, port_p_mod =est_ms[[iteration_mx]]$port_p_mod,  
                         p_LM_resid_corr =  est_ms[[iteration_mx]]$p_LM_resid_corr ,B_se = est_ms[[iteration_mx]]$B_se, Lambda_se = est_ms[[iteration_mx]]$Lambda_se, 
                         Var_se = est_ms[[iteration_mx]]$Var_se,P_se =est_ms[[iteration_mx]]$P_se, beta_se =est_ms[[iteration_mx]]$beta_se, lambda_nu_se =est_ms[[iteration_mx]]$lambda_se, 
                         beta =est_ms[[iteration_mx]]$beta, lambda_nu =est_ms[[iteration_mx]]$lambda_nu,convergence=est_ms[[iteration_mx]]$convergence)
    }
    
    return(estimation)
  } else if (spec$unres){
    RandNumbers<- 1/1000*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K)
    
    est_ms <- estimate_ms_unres(A1,A2,Samp,K,y,Z,spec, Tolvalue, RandNumbers, code, log_path, verbose, start_val)
    est_ms$num_param <-  2*K + spec$lag*K^2 + (spec$states-1)*(spec$states)  + K*(K+1)/2*(spec$states) 
    est_ms$selec_crit <- selec_crit_calc(est_ms$logL[1],K ,Samp,spec$lag, est_ms$num_param)
    return(est_ms)  
  }
  
} # end of function

