bootstrap_proxy <- function(est_proxy, lag_length, spec, Samp,K, y, y_start, date_list,log_path){
  
  y_true <-  y
  horizon <- spec$horizon
  rep <-  spec$rep
  B <- est_proxy$B
  Theta <- est_proxy$Theta
  u_est <- est_proxy$u_est
  instrument <- as.matrix(spec$instrument)
  # u_est <- as.matrix(read.table("u_est.csv", sep=","))
  # create empty arrays to hold the data
  Z_star_array <- array(data=NA, dim=c( (Samp-lag_length),(K*lag_length+1),rep))
  u_star_array <- array(data=NA, dim=c((Samp-lag_length),K, rep ))
  y_star_array <- array(data=NA, dim=c((Samp-lag_length),K,rep))
  # array if instrument approach is supposed to be used 
  length_dif <- dim(as.matrix(spec$instrument))[1]-dim(u_est)[1]
  if (length_dif > 0){
    instrument <- as.matrix(spec$instrument[-c(1:length_dif)])
  }
  instr_star_array <- array(data=NA, dim=c( dim(instrument)[1],dim(instrument)[2],rep))
  
  for (n in 1:rep){ #rep
    boot_data <- create_boot_data_proxy(y_true, y_start, lag_length, spec, Samp, Theta, K, u_est, date_list, instrument)  
    Z_star_array[,,n] <- boot_data[[2]]
    y_star_array[,,n] <- boot_data[[1]]
    u_star_array[,,n] <- boot_data[[3]]
    instr_star_array[,,n] <- boot_data[[4]]
  }
  
  #re-run the models, with estimated parameters (no grid search)
  results_boot <- vector(mode = "list", length = rep)
  
  # re-run the model 
  # ------------------------------------------------------------------------------------
  #########################################
  # VAR: Exogenous changes in covariances #
  #########################################
  # ------------------------------------------------------------------------------------
  boot_imp_complete <- array(data=NA, dim=c(rep,horizon+1 ,K))
  boot_imp_cum_complete <- array(data=NA, dim=c(rep,horizon+1 ,K))
  results_boot <-   foreach(n=(1:rep),.verbose =FALSE,  .export=ls(.GlobalEnv), .packages=c("nloptr","matrixcalc", "pracma","miscTools", "MASS", "NlcOptim")) %dopar% { # loop through the grid
    #-------------------------------- data ------------------------------------------------
    y  <- as.matrix(y_star_array[,,n])
    Z  <- as.matrix(Z_star_array[,,n])
    spec$instrument <- instr_star_array[,,n]
    #-----------------------------ML-estimation----------------------------------------------
    cat(paste("Bootstrap iteration:", n ,"\r\n"),  file=log_path, append=TRUE)
    results_boot[[n]] <-  proxy_var_boot(spec, y, Z, Samp, K, y_start, date_list)
  }
  
  horizon_2 <- dim(boot_imp_complete[,,1])[2]
  no_shocks <- K
  # create empty arrays  
  lower_5 <- array(NA, dim=c(no_shocks, horizon_2))
  upper_5 <- array(NA, dim=c(no_shocks, horizon_2))
  
  lower_10 <- array(NA, dim=c(no_shocks, horizon_2))
  upper_10 <- array(NA, dim=c(no_shocks, horizon_2))
  lower_68 <- array(NA, dim=c(no_shocks, horizon_2))
  upper_68 <- array(NA, dim=c(no_shocks, horizon_2))
  lower_5_cum <-array(NA, dim=c(no_shocks, horizon_2))
  upper_5_cum <- array(NA, dim=c(no_shocks, horizon_2))
  lower_10_cum <-array(NA, dim=c(no_shocks, horizon_2))
  upper_10_cum <- array(NA, dim=c(no_shocks, horizon_2))
  
  lower_68_cum <- array(NA, dim=c(no_shocks, horizon_2))
  upper_68_cum <- array(NA, dim=c(no_shocks, horizon_2))
  interval1 <- 0.025
  interval2 <- 0.05
  interval3 <- 0.16
  
  for (n in 1:rep){
    for (l in 1:K){
      boot_imp_complete[n,,l] <- results_boot[[n]]$imp_rep[l,]
      boot_imp_cum_complete[n,,l] <- results_boot[[n]]$imp_rep_cum[l,]
    } # end of loop through l (number of shocks)
  } # end of loop through n (rep)
  
  # sort the bootstrapped impulse responses and order them by size
  sorted_impulse <- boot_imp_complete
  sorted_impulse_cum <- boot_imp_cum_complete
  
  
  # order by size 
  for (i in 1:no_shocks){
    sorted_impulse[,,i] <- apply(boot_imp_complete[,,i], 2, sort, decreasing=TRUE)
    sorted_impulse_cum[,,i] <- apply(boot_imp_cum_complete[,,i], 2, sort, decreasing=TRUE)
    lower_5[i,] <- sorted_impulse[ceiling((1-interval1)*rep),,i]
    upper_5[i,] <- sorted_impulse[floor(interval1*rep),,i]
    lower_10[i,] <- sorted_impulse[ceiling((1-interval2)*rep),,i]
    upper_10[i,] <- sorted_impulse[floor(interval2*rep),,i]
    lower_68[i,] <- sorted_impulse[ceiling((1-interval3)*rep),,i]
    upper_68[i,] <- sorted_impulse[floor(interval3*rep),,i]
    lower_5_cum[i,] <- sorted_impulse_cum[ceiling((1-interval1)*rep),,i]
    upper_5_cum[i,] <- sorted_impulse_cum[floor(interval1*rep),,i]
    lower_10_cum[i,] <- sorted_impulse_cum[ceiling((1-interval2)*rep),,i]
    upper_10_cum[i,] <- sorted_impulse_cum[floor(interval2*rep),,i]
    
    lower_68_cum[i,] <- sorted_impulse_cum[ceiling((1-interval3)*rep),,i]
    upper_68_cum[i,] <- sorted_impulse_cum[floor(interval3*rep),,i]
  }
  
  out <-list(upper_int5 = upper_5, lower_int5 =lower_5,upper_int10 = upper_10, lower_int10 =lower_10,upper_int68 = upper_68, lower_int68 =lower_68, sorted_impulse=sorted_impulse, upper_int5_cum = upper_5_cum, lower_int5_cum =lower_5_cum,upper_int10_cum = upper_10_cum, lower_int10_cum =lower_10_cum,upper_int68_cum = upper_68_cum, lower_int68_cum =lower_68_cum, sorted_impulse_cum=sorted_impulse_cum)
  return(out)
}

