calc_imp_response_proxy <- function(B, Theta, horizon, K, spec, lag_length){
  # extract the intercept from the matrix of estimators
  int <- Theta[1:K,1]
  # Write in VAR(1) representation (Companion form)
  nu <- c(int, vector(mode="numeric", length= K*lag_length-K) )
  if (lag_length == 1){
    A_var <- Theta[,2:dim(Theta)[2]]
  } else if (lag_length > 1){
    diag_help <- cbind(diag(K*lag_length-K), matrix(0,K*lag_length-K,K))
    A_var <- rbind(Theta[,2:dim(Theta)[2]], diag_help)
  } 
  # calculate the characteristical polynomial
  char_poly <-  diag(K*lag_length)-A_var
  # caluclate the mean
  mean <- (solve(char_poly)%*%nu)[1:K]
  # calculate J, an extractor matrix
  J <- cbind(diag(K), matrix(0,K, K*lag_length -K))
  # derive the MA-coefficients (phi) by multiplication of J and A^i (Lütkepohl page 18)
  phi <- array(NA, dim=c(K,K,horizon+1))
  for (i in 0:horizon){
    phi_step <- J%*%matrix.power(as.matrix(A_var),i)%*%t(J)
    if (i==0){
      phi[,,(i+1)] <- phi_step
    } else if (i >0){
      phi[,,(i+1)] <- phi_step 
    }
  }
  # calculate the response coefficient of a (one standard deviation) structural shock (Phi = phi*B*I)
  Phi <- array(NA, dim=c(K,K,horizon+1))
  for (i in 0:horizon){
    # Phi_step <-  phi[,,(i+1)]%*%B%*%diag(rep(1, K))
    Phi_step <-  phi[,,(i+1)]%*%B
    
    if (i==0){
      Phi[,,(i+1)] <- Phi_step
    } else if (i >0){
      Phi[,,(i+1)] <-Phi_step
    }
  }
  # calculate the impulse responses
  impulse <- array(NA,  dim=c(K*K, horizon+1))
  impulse_cum <- array(NA,dim=c(K*K, horizon+1))
  for (i in 0:horizon){
    impulse[,(i+1)] <- c(Phi[,,(i+1)]%*%diag(rep(1,K))) 
  }
  # calculate the cumulative impulse responses
  
  for (j in 1:(K^2)){
    for (i in 0:horizon){
      impulse_cum[,(i+1)] <- rowSums(as.matrix(impulse[,(1:(i+1))]))
    }
  }
  
  out <- list(impulse = impulse, impulse_cum = impulse_cum, Phi = Phi, ma_coef = phi, process_mean=mean)
  
  return(out)
}