proxy_var <- function(spec, y, Z,y_start,  Samp, K, date_list =c()){
  
  y_reg <- c(t(y)) # transpose and vectorize data 
  
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  
  Theta <-  solve(A1)%*%A2
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, K, K*spec$lag+1)
  u_est <- residuals3(y,Theta_mat,Z)  
  Theta <- Theta_mat  
  
  Sigma <- (Samp-(1+spec$lag*K))^-1*t(u_est)%*%u_est  
  ##################
  # since identification relies on instrument period only (might be shorter!) adjust u_est
  #####
  row_start <- which(grepl(date_list[[2]][1], date_list[[1]]))
  row_end <- which(grepl(date_list[[2]][length(date_list[[2]])], date_list[[1]]))
  Samp_adj <- length(row_start:row_end)
  u_est_adj <- u_est[(row_start):(row_end-spec$lag),]
  Sigma_adj <- (Samp_adj-spec$lag-(1+spec$lag*K))^-1*t(u_est_adj)%*%u_est_adj  
  
  impulse_vecs <- impulse_vecs(u_est_adj, spec, Samp_adj, K)
  
  B <- matrix(0, K,K)
  # the ordering of the columns of B is arbitrary, thus we can just put it in the first column of B-matrix
  B[,1] <- impulse_vecs$abs_impulse
  if (any(is.na(B))) {
    B <- diag(K) 
  }
  
  # accordingly, we need to solve for the first row of A-matrix
  a <-  solve_for_a(impulse_vecs$abs_impulse, Sigma)
  # the respective structural shock
  struc_shock <- c()
  for (i in 1:(Samp-spec$lag)){
    struc_shock[i] <- a%*%u_est[i,]
  }
  # generate random impact in all other columns
  B[,(2:K)] <- rnorm((K-1)*(K),0, 1)
  
  if (spec$sim ==TRUE){
    if (sign(B[1,1]) == -1){
      B[,1] <- -1*B[,1]   
    } 
  }
  
  struc_shock <- cbind(struc_shock, matrix(rnorm((K-1)*length(struc_shock),0, 1), ncol=(K-1)))
  st_struc_shock <- u_stand_var(Samp,K, y, Z, spec$lag, estimation <- list(Sigma=Sigma), u_est)
  imp_rep <- calc_imp_response_proxy(B, Theta, dim(u_est)[1]-1, K, spec, spec$lag)
  imp_rep$impulse <- imp_rep$impulse[1:K,]
  imp_rep$impulse_cum <- imp_rep$impulse_cum[1:K,]
  
  
  num_param <-  K + spec$lag*K^2 + K
  # selection criteria
  selec_crit <- selec_crit_calc_proxy(Sigma,K ,Samp,spec$lag, num_param)
  
  variance_decomp <- calc_variance_decomp_proxy(imp_rep$Phi, struc_shock, K,  spec,  spec$horizon)
  
  hist_decomp <- calc_hist_decomp_proxy(imp_rep$Phi, struc_shock, K, horizon=dim(u_est)[1]-1)
  
  est_proxy <- list(B = B, a = a, Theta=Theta, u_est = u_est, Sigma = Sigma, rel_imp = impulse_vecs$rel_impulse , f_test = impulse_vecs$f_test, f_robust = impulse_vecs$f_robust, struc_shock = struc_shock, st_struc_shock = st_struc_shock$u_st, imp_rep = imp_rep$impulse, imp_rep_cum = imp_rep$impulse_cum, process_mean = imp_rep$process_mean, Phi = imp_rep$Phi, hist_decomp = hist_decomp$hist_decomp, hist_decomp_contrib= hist_decomp$hist_decomp_contrib, variance_decomp=variance_decomp)
  
  if (spec$boot== "yes"){
    cat(paste("Proxy VAR: Starting bootstrap... Check Log-File for Progress! \r\n"))
    
    cat(paste("Proxy VAR: Starting bootstrap\r\n"),  file=log_path, append=TRUE)
    boot_int <- bootstrap_proxy(est_proxy, lag_length, spec, Samp,K, y, y_start, date_list,log_path)
    est_proxy <- list(B = est_proxy$B, a = est_proxy$a, Theta=est_proxy$Theta, u_est = est_proxy$u_est, Sigma = est_proxy$Sigma, f_test = est_proxy$f_test, f_robust = est_proxy$f_robust,num_param = num_param, AIC = selec_crit$AIC,HQ = selec_crit$HQ,SC = selec_crit$SC, struc_shock = est_proxy$struc_shock, st_struc_shock = est_proxy$st_struc_shock, imp_rep = est_proxy$imp_rep, imp_rep_cum = est_proxy$imp_rep_cum, process_mean = est_proxy$process_mean, Phi = est_proxy$Phi, hist_decomp = est_proxy$hist_decomp, hist_decomp_contrib= est_proxy$hist_decomp_contrib, variance_decomp=est_proxy$variance_decomp, upper_int5 = boot_int$upper_int5, lower_int5 = boot_int$lower_int5, upper_int10 = boot_int$upper_int10, lower_int10 = boot_int$lower_int10,upper_int68 = boot_int$upper_int68, lower_int68 = boot_int$lower_int68,  upper_int5_cum = boot_int$upper_int5_cum, lower_int5_cum = boot_int$lower_int5_cum, upper_int10_cum = boot_int$upper_int10_cum, lower_int10_cum = boot_int$lower_int10_cum,upper_int68_cum = boot_int$upper_int68_cum, lower_int68_cum = boot_int$lower_int68_cum)
    cat(paste("Proxy VAR: Bootstrap completed. \r\n"))  
    
  }
  
  return(est_proxy)
  
}
