proxy_var_boot <- function(spec, y, Z, Samp, K, y_start, date_list){
  y_reg <- c(t(y)) # transpose and vectorize data 
  A1 <- kronecker(t(Z)%*%Z,diag(K))
  A2 <- kronecker(t(Z), diag(K))%*%y_reg
  
  Theta <-  solve(A1)%*%A2
  # get residuals to calculate initial B
  Theta_mat <- matrix(Theta, K, K*spec$lag+1)
  u_est <- residuals3(y,Theta_mat,Z)  
  Theta <- Theta_mat  
  
  Sigma <- (Samp-(1+spec$lag*K))^-1*t(u_est)%*%u_est  
  ##################
  # since identification relies on instrument period only (might be shorter!) adjust u_est
  #####
  row_start <- which(grepl(date_list[[2]][1], date_list[[1]]))
  row_end <- which(grepl(min(date_list[[2]][length(date_list[[2]])],date_list[[1]][length(date_list[[1]])]) , date_list[[1]]))
  
  Samp_adj <- length(row_start:row_end)
  u_est_adj <- u_est[(row_start):(row_end-spec$lag),] 
  Sigma_adj <- (Samp_adj-spec$lag-(1+spec$lag*K))^-1*t(u_est_adj)%*%u_est_adj  
  
  impulse_vecs <- impulse_vecs(u_est_adj, spec, Samp_adj, K)
  B <- matrix(0, K,K)
  # the ordering of the columns of B is arbitrary, thus we can just put it in the first column of B-matrix
  B[,1] <- impulse_vecs$abs_impulse
  
  
  # accordingly, we need to solve for the first row of A-matrix
  a <-  solve_for_a(impulse_vecs$abs_impulse, Sigma)
  # the respective structural shock
  struc_shock <- c()
  for (i in 1:(Samp-spec$lag)){
    struc_shock[i] <- a%*%u_est[i,]
  }
  # generate random impact in all other columns
  B[,(2:K)] <- rnorm((K-1)*(K),0, 1)
  # Normalize B-matrix to have positive mian diagonal
  # Using Likelihood Preserving normalization (Waggoner/Zha (2003)):
  
  if (B[2,1] < 0){
    B[,1] <-  -1*B[,1]
  }
  
  struc_shock <- cbind(struc_shock, matrix(rnorm((K-1)*length(struc_shock),0, 1), ncol=(K-1)))
  st_struc_shock <- u_stand_var(Samp,K, y, Z, spec$lag, estimation <- list(Sigma=Sigma), u_est)
  imp_rep <- calc_imp_response_proxy(B, Theta, spec$horizon, K, spec, spec$lag)
  imp_rep$impulse <- imp_rep$impulse[1:K,]
  imp_rep$impulse_cum <- imp_rep$impulse_cum[1:K,]
  
  prox_var_boot <- list(B = B, a = a, Theta=Theta, u_est = u_est, Sigma = Sigma, struc_shock = struc_shock, st_struc_shock = st_struc_shock$u_st, imp_rep = imp_rep$impulse, imp_rep_cum = imp_rep$impulse_cum)
}
