data_mswitch <- function(Samp, coef_true, mean, P_true, B_true, Lambda_true, lambda_nu, beta_true, spec){
  #               ############################# 
  #               # Generate the Markov Chain #
  #               #############################
  
  if (dim(P_true)[2]==2){
    current_state <- draw_Mchain_2(P_true, Samp)
  } else if (dim(P_true)[2]==3){
    current_state <- draw_Mchain_3(P_true, Samp)
  }
  
  # plot(current_state, type='l')
  Sigma_st  <- vector(mode="list", length=Samp)
  noise_variance <- vector(mode="list", length=Samp)
  eps <- matrix(NA, Samp, K)
  noise <- vector(mode="list", length=Samp)
  for (i in 1:Samp){
    eps[i,] <-   rmvnorm(1, c(mean), diag(rev(diag(Lambda_true[[current_state[i]]]))))
    noise[[i]] <- rmvnorm(1, rep(0, spec$ins_dim), lambda_nu[[current_state[i]]])
  }
  eps <- eps[,c(K:1)]
  # generate the instrument
  instrument <- matrix(NA, Samp, spec$ins_dim)
  for (i in 1:Samp){
    instrument[i,] <- t(beta_true %*% eps[i,]) + noise[[i]]
  }
  if (spec$set_zero == TRUE){
    instrument[instrument %in% c(head(sort(abs(instrument[current_state==1])),ceiling(dim(instrument)[1]*(sum(current_state==1)/Samp)*spec$share0)),head(sort(abs(instrument[current_state==2])),floor(dim(instrument)[1]*(sum(current_state==2)/Samp)*spec$share0)), 
                                 -head(sort(abs(instrument[current_state==1])),ceiling(dim(instrument)[1]*(sum(current_state==1)/Samp)*spec$share0)),-head(sort(abs(instrument[current_state==2])),floor(dim(instrument)[1]*(sum(current_state==2)/Samp)*spec$share0)))] = 0
    
  }
  # generate the residuals that depend on Sigma_st
  if (spec$fail_constB == TRUE){
    B_vary <- list(B_true, B_true+ 0.05*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K), B_true+ 0.05*matrix(rnorm(length(matrix(0,K,K)),0,1), K,K))
    u <- matrix(NA, Samp, K)
    for (i in 1:Samp){
      u[i,] <-   t(B_vary[[current_state[i]]]%*%eps[i,])
    }
  } else {
    u <- matrix(NA, Samp, K)
    for (i in 1:Samp){
      u[i,] <-   t(B_true%*%eps[i,])
    }
  }
  
  #               ####################### 
  #               # Generate the data   #
  #               #######################
  spec$deter <- as.matrix(rep(1,Samp))
  # generate data recursively
  Theta <-  matrix(unlist(coef_true), nrow=K) # build coefficient matrix
  mean_ts <- (solve(diag(K*spec$lag) - var_companion(Theta, K, spec)$A_var)%*% c(coef_true$intercept,rep(0, K*(spec$lag-1))))[1:K,]
  y <-  rbind(matrix(rep(mean_ts, each=spec$lag), ncol=K),matrix(0,dim(u)[1] ,dim(u)[2]))
  for  (r in (spec$lag+1):(Samp+spec$lag)){
    y[r,] <- Theta%*%c(1,c(t(y[(r-1):(r-spec$lag),]))) + u[r-spec$lag,]
  }
  # cut starting values
  y <- y[-c(1:spec$lag),]
  
  out <- list(y=y,resid=u, states_dgp = current_state, struc_shocks=eps, instrument=instrument)
}# end of function
