data_smooth <- function(Samp, K, coef_true, mean, P_true, B_true, Lambda_true, lambda_nu, beta_true,  spec){
  #  time is transition variable
  if (spec$gen_ST == TRUE){
    tr_var <-  seq(1,Samp,1)
    gamma <- -sqrt(Samp)/10
    c <- 0.5*Samp
  } else if (spec$gen_ST_IP == TRUE){
    IP <- movavg(as.matrix(readRDS("../simulation/organization_code/industrial_production.Rds"))[1:Samp], 12, type = "s")
    l = 25
    row_block_mat <-  length(l:Samp)
    # draw rows of block matrix with replacement that are used to construct series of residuals later
    row_draw <- sample(c(1:row_block_mat), ceiling(Samp/l), replace = TRUE )    
    
    block_IP <- c()
    tr_var <- c()
    counter <-  0
    for (i in l:Samp){
      # i=1
      counter <-  counter + 1
      # matrix with sequences of residuals of length s
      block_IP <- rbind(block_IP, IP[counter: (l+(counter-1))] )
      
    }
    col_means_block <- apply(block_IP, 2, mean)
    block_IP_demean <- sweep(block_IP, 2, col_means_block)
    
    # construct u_star by using the demeaned residuals (with means of the rows of the block matrix)
    tr_var <- cbind(tr_var,c(t(block_IP_demean[row_draw,]))[1:Samp])
    c <- 0
    gamma <- 2.5
  }
  
  #  generate the transition function basede on transition variable and parameters
  tr_func <- as.vector((1+exp(-exp(gamma)*(tr_var-c)))^-1)
  # create empty vectors for variance and residuals
  eps <- matrix(NA, Samp, K)
  noise <- vector(mode="list", Samp)
  #  generate the expectation of variance as sum of transition function and variances, generate the residuals based on variances
  for(i in 1:Samp){
    eps[i,] <-  rmvnorm(1,mean,as.matrix((1-tr_func[i])*Lambda_true[[1]]+tr_func[i]*Lambda_true[[2]]))
    noise[[i]] <- rmvnorm(1, rep(0, spec$ins_dim), as.matrix((1-tr_func[i])*lambda_nu[[1]]+tr_func[i]*lambda_nu[[2]]))
  } 
  # generate the instrument
  instrument <- matrix(NA, Samp, spec$ins_dim)
  for (i in 1:Samp){
    instrument[i,] <- t(beta_true %*% eps[i,]) + noise[[i]]
  }
  
  # compute the residuals
  u <- matrix(NA, Samp, K)
  for (i in 1:Samp){
    u[i,] <-   t(B_true%*%eps[i,])
  }
  
  # deterministic variables in model
  spec$deter <- as.matrix(rep(1,Samp))
  # generate data recursively
  Theta <-  matrix(unlist(coef_true), nrow=K) # build coefficient matrix
  mean_ts <- (solve(diag(K*spec$lag) - var_companion(Theta, K, spec)$A_var)%*% c(coef_true$intercept,rep(0, K*(spec$lag-1))))[1:K,]
  y <-  rbind(matrix(rep(mean_ts, each=spec$lag), ncol=K),matrix(0,dim(u)[1] ,dim(u)[2]))
  for  (r in (spec$lag+1):(Samp+spec$lag)){
    y[r,] <- Theta%*%c(1,c(t(y[(r-1):(r-spec$lag),]))) + u[r-spec$lag,]
  }
  # cut starting values
  y <- y[-c(1:spec$lag),]
  out <- list(y=y,resid=u, states_dgp = tr_func, struc_shocks=eps, instrument=instrument)
  
}# end of function
